import gzip
import http.client
import sys
import time
from _typeshed import SupportsRead, SupportsWrite
from datetime import datetime
from io import BytesIO
from types import TracebackType
from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Protocol, Tuple, Type, Union, overload
from typing_extensions import Literal

class _SupportsTimeTuple(Protocol):
    def timetuple(self) -> time.struct_time: ...

_DateTimeComparable = Union[DateTime, datetime, str, _SupportsTimeTuple]
_Marshallable = Union[None, bool, int, float, str, bytes, Tuple[Any, ...], List[Any], Dict[Any, Any], datetime, DateTime, Binary]
_XMLDate = Union[int, datetime, Tuple[int, ...], time.struct_time]
_HostType = Union[Tuple[str, Dict[str, str]], str]

def escape(s: str) -> str: ...  # undocumented

MAXINT: int  # undocumented
MININT: int  # undocumented

PARSE_ERROR: int  # undocumented
SERVER_ERROR: int  # undocumented
APPLICATION_ERROR: int  # undocumented
SYSTEM_ERROR: int  # undocumented
TRANSPORT_ERROR: int  # undocumented

NOT_WELLFORMED_ERROR: int  # undocumented
UNSUPPORTED_ENCODING: int  # undocumented
INVALID_ENCODING_CHAR: int  # undocumented
INVALID_XMLRPC: int  # undocumented
METHOD_NOT_FOUND: int  # undocumented
INVALID_METHOD_PARAMS: int  # undocumented
INTERNAL_ERROR: int  # undocumented

class Error(Exception): ...

class ProtocolError(Error):

    url: str
    errcode: int
    errmsg: str
    headers: Dict[str, str]
    def __init__(self, url: str, errcode: int, errmsg: str, headers: Dict[str, str]) -> None: ...

class ResponseError(Error): ...

class Fault(Error):

    faultCode: int
    faultString: str
    def __init__(self, faultCode: int, faultString: str, **extra: Any) -> None: ...

boolean = bool
Boolean = bool

def _iso8601_format(value: datetime) -> str: ...  # undocumented
def _strftime(value: _XMLDate) -> str: ...  # undocumented

class DateTime:

    value: str  # undocumented
    def __init__(self, value: Union[int, str, datetime, time.struct_time, Tuple[int, ...]] = ...) -> None: ...
    def __lt__(self, other: _DateTimeComparable) -> bool: ...
    def __le__(self, other: _DateTimeComparable) -> bool: ...
    def __gt__(self, other: _DateTimeComparable) -> bool: ...
    def __ge__(self, other: _DateTimeComparable) -> bool: ...
    def __eq__(self, other: _DateTimeComparable) -> bool: ...  # type: ignore
    def make_comparable(self, other: _DateTimeComparable) -> Tuple[str, str]: ...  # undocumented
    def timetuple(self) -> time.struct_time: ...  # undocumented
    def decode(self, data: Any) -> None: ...
    def encode(self, out: SupportsWrite[str]) -> None: ...

def _datetime(data: Any) -> DateTime: ...  # undocumented
def _datetime_type(data: str) -> datetime: ...  # undocumented

class Binary:

    data: bytes
    def __init__(self, data: Optional[bytes] = ...) -> None: ...
    def decode(self, data: bytes) -> None: ...
    def encode(self, out: SupportsWrite[str]) -> None: ...

def _binary(data: bytes) -> Binary: ...  # undocumented

WRAPPERS: Tuple[Type[DateTime], Type[Binary]]  # undocumented

class ExpatParser:  # undocumented
    def __init__(self, target: Unmarshaller) -> None: ...
    def feed(self, data: Union[str, bytes]) -> None: ...
    def close(self) -> None: ...

class Marshaller:

    dispatch: Dict[
        Type[Any], Callable[[Marshaller, Any, Callable[[str], Any]], None]
    ] = ...  # TODO: Replace 'Any' with some kind of binding

    memo: Dict[Any, None]
    data: None
    encoding: Optional[str]
    allow_none: bool
    def __init__(self, encoding: Optional[str] = ..., allow_none: bool = ...) -> None: ...
    def dumps(self, values: Union[Fault, Iterable[_Marshallable]]) -> str: ...
    def __dump(self, value: Union[_Marshallable], write: Callable[[str], Any]) -> None: ...  # undocumented
    def dump_nil(self, value: None, write: Callable[[str], Any]) -> None: ...
    def dump_bool(self, value: bool, write: Callable[[str], Any]) -> None: ...
    def dump_long(self, value: int, write: Callable[[str], Any]) -> None: ...
    def dump_int(self, value: int, write: Callable[[str], Any]) -> None: ...
    def dump_double(self, value: float, write: Callable[[str], Any]) -> None: ...
    def dump_unicode(self, value: str, write: Callable[[str], Any], escape: Callable[[str], str] = ...) -> None: ...
    def dump_bytes(self, value: bytes, write: Callable[[str], Any]) -> None: ...
    def dump_array(self, value: Iterable[_Marshallable], write: Callable[[str], Any]) -> None: ...
    def dump_struct(
        self, value: Mapping[str, _Marshallable], write: Callable[[str], Any], escape: Callable[[str], str] = ...
    ) -> None: ...
    def dump_datetime(self, value: _XMLDate, write: Callable[[str], Any]) -> None: ...
    def dump_instance(self, value: object, write: Callable[[str], Any]) -> None: ...

class Unmarshaller:

    dispatch: Dict[str, Callable[[Unmarshaller, str], None]] = ...

    _type: Optional[str]
    _stack: List[_Marshallable]
    _marks: List[int]
    _data: List[str]
    _value: bool
    _methodname: Optional[str]
    _encoding: str
    append: Callable[[Any], None]
    _use_datetime: bool
    _use_builtin_types: bool
    def __init__(self, use_datetime: bool = ..., use_builtin_types: bool = ...) -> None: ...
    def close(self) -> Tuple[_Marshallable, ...]: ...
    def getmethodname(self) -> Optional[str]: ...
    def xml(self, encoding: str, standalone: Any) -> None: ...  # Standalone is ignored
    def start(self, tag: str, attrs: Dict[str, str]) -> None: ...
    def data(self, text: str) -> None: ...
    def end(self, tag: str) -> None: ...
    def end_dispatch(self, tag: str, data: str) -> None: ...
    def end_nil(self, data: str) -> None: ...
    def end_boolean(self, data: str) -> None: ...
    def end_int(self, data: str) -> None: ...
    def end_double(self, data: str) -> None: ...
    def end_bigdecimal(self, data: str) -> None: ...
    def end_string(self, data: str) -> None: ...
    def end_array(self, data: str) -> None: ...
    def end_struct(self, data: str) -> None: ...
    def end_base64(self, data: str) -> None: ...
    def end_dateTime(self, data: str) -> None: ...
    def end_value(self, data: str) -> None: ...
    def end_params(self, data: str) -> None: ...
    def end_fault(self, data: str) -> None: ...
    def end_methodName(self, data: str) -> None: ...

class _MultiCallMethod:  # undocumented

    __call_list: List[Tuple[str, Tuple[_Marshallable, ...]]]
    __name: str
    def __init__(self, call_list: List[Tuple[str, _Marshallable]], name: str) -> None: ...
    def __getattr__(self, name: str) -> _MultiCallMethod: ...
    def __call__(self, *args: _Marshallable) -> None: ...

class MultiCallIterator:  # undocumented

    results: List[List[_Marshallable]]
    def __init__(self, results: List[List[_Marshallable]]) -> None: ...
    def __getitem__(self, i: int) -> _Marshallable: ...

class MultiCall:

    __server: ServerProxy
    __call_list: List[Tuple[str, Tuple[_Marshallable, ...]]]
    def __init__(self, server: ServerProxy) -> None: ...
    def __getattr__(self, item: str) -> _MultiCallMethod: ...
    def __call__(self) -> MultiCallIterator: ...

# A little white lie
FastMarshaller: Optional[Marshaller]
FastParser: Optional[ExpatParser]
FastUnmarshaller: Optional[Unmarshaller]

def getparser(use_datetime: bool = ..., use_builtin_types: bool = ...) -> Tuple[ExpatParser, Unmarshaller]: ...
def dumps(
    params: Union[Fault, Tuple[_Marshallable, ...]],
    methodname: Optional[str] = ...,
    methodresponse: Optional[bool] = ...,
    encoding: Optional[str] = ...,
    allow_none: bool = ...,
) -> str: ...
def loads(
    data: str, use_datetime: bool = ..., use_builtin_types: bool = ...
) -> Tuple[Tuple[_Marshallable, ...], Optional[str]]: ...
def gzip_encode(data: bytes) -> bytes: ...  # undocumented
def gzip_decode(data: bytes, max_decode: int = ...) -> bytes: ...  # undocumented

class GzipDecodedResponse(gzip.GzipFile):  # undocumented

    io: BytesIO
    def __init__(self, response: SupportsRead[bytes]) -> None: ...
    def close(self) -> None: ...

class _Method:  # undocumented

    __send: Callable[[str, Tuple[_Marshallable, ...]], _Marshallable]
    __name: str
    def __init__(self, send: Callable[[str, Tuple[_Marshallable, ...]], _Marshallable], name: str) -> None: ...
    def __getattr__(self, name: str) -> _Method: ...
    def __call__(self, *args: _Marshallable) -> _Marshallable: ...

class Transport:

    user_agent: str = ...
    accept_gzip_encoding: bool = ...
    encode_threshold: Optional[int] = ...

    _use_datetime: bool
    _use_builtin_types: bool
    _connection: Tuple[Optional[_HostType], Optional[http.client.HTTPConnection]]
    _headers: List[Tuple[str, str]]
    _extra_headers: List[Tuple[str, str]]

    if sys.version_info >= (3, 8):
        def __init__(
            self, use_datetime: bool = ..., use_builtin_types: bool = ..., *, headers: Iterable[Tuple[str, str]] = ...
        ) -> None: ...
    else:
        def __init__(self, use_datetime: bool = ..., use_builtin_types: bool = ...) -> None: ...
    def request(self, host: _HostType, handler: str, request_body: bytes, verbose: bool = ...) -> Tuple[_Marshallable, ...]: ...
    def single_request(
        self, host: _HostType, handler: str, request_body: bytes, verbose: bool = ...
    ) -> Tuple[_Marshallable, ...]: ...
    def getparser(self) -> Tuple[ExpatParser, Unmarshaller]: ...
    def get_host_info(self, host: _HostType) -> Tuple[str, List[Tuple[str, str]], Dict[str, str]]: ...
    def make_connection(self, host: _HostType) -> http.client.HTTPConnection: ...
    def close(self) -> None: ...
    def send_request(self, host: _HostType, handler: str, request_body: bytes, debug: bool) -> http.client.HTTPConnection: ...
    def send_headers(self, connection: http.client.HTTPConnection, headers: List[Tuple[str, str]]) -> None: ...
    def send_content(self, connection: http.client.HTTPConnection, request_body: bytes) -> None: ...
    def parse_response(self, response: http.client.HTTPResponse) -> Tuple[_Marshallable, ...]: ...

class SafeTransport(Transport):

    if sys.version_info >= (3, 8):
        def __init__(
            self,
            use_datetime: bool = ...,
            use_builtin_types: bool = ...,
            *,
            headers: Iterable[Tuple[str, str]] = ...,
            context: Optional[Any] = ...,
        ) -> None: ...
    else:
        def __init__(self, use_datetime: bool = ..., use_builtin_types: bool = ..., *, context: Optional[Any] = ...) -> None: ...
    def make_connection(self, host: _HostType) -> http.client.HTTPSConnection: ...

class ServerProxy:

    __host: str
    __handler: str
    __transport: Transport
    __encoding: str
    __verbose: bool
    __allow_none: bool

    if sys.version_info >= (3, 8):
        def __init__(
            self,
            uri: str,
            transport: Optional[Transport] = ...,
            encoding: Optional[str] = ...,
            verbose: bool = ...,
            allow_none: bool = ...,
            use_datetime: bool = ...,
            use_builtin_types: bool = ...,
            *,
            headers: Iterable[Tuple[str, str]] = ...,
            context: Optional[Any] = ...,
        ) -> None: ...
    else:
        def __init__(
            self,
            uri: str,
            transport: Optional[Transport] = ...,
            encoding: Optional[str] = ...,
            verbose: bool = ...,
            allow_none: bool = ...,
            use_datetime: bool = ...,
            use_builtin_types: bool = ...,
            *,
            context: Optional[Any] = ...,
        ) -> None: ...
    def __getattr__(self, name: str) -> _Method: ...
    @overload
    def __call__(self, attr: Literal["close"]) -> Callable[[], None]: ...
    @overload
    def __call__(self, attr: Literal["transport"]) -> Transport: ...
    @overload
    def __call__(self, attr: str) -> Union[Callable[[], None], Transport]: ...
    def __enter__(self) -> ServerProxy: ...
    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> None: ...
    def __close(self) -> None: ...  # undocumented
    def __request(self, methodname: str, params: Tuple[_Marshallable, ...]) -> Tuple[_Marshallable, ...]: ...  # undocumented

Server = ServerProxy



import humps
from pydantic import BaseModel


class PydanticBaseDjango(BaseModel):
    class Config:
        # no additional properties
        extra = "forbid"

        # Allow initialization via attribute names and aliases
        allow_population_by_field_name = True

        # auto generate camelized aliases
        alias_generator = humps.camelize


__all__ = ["PydanticBaseDjango"]



from typing import Any

from django.core.management import BaseCommand

class Command(BaseCommand):
    shells: list[str]
    def ipython(self, options: Any) -> None: ...
    def bpython(self, options: Any) -> None: ...
    def python(self, options: Any) -> None: ...



from collections.abc import Sequence
from typing import Any

from django.apps.config import AppConfig
from django.core.checks.messages import CheckMessage, Error, Warning

SECRET_KEY_MIN_LENGTH: int
SECRET_KEY_MIN_UNIQUE_CHARACTERS: int
W001: Warning
W002: Warning
W004: Warning
W005: Warning
W006: Warning
W007: Warning
W008: Warning
W009: Warning
W018: Warning
W019: Warning
W020: Warning
W021: Warning
W022: Warning
E023: Error
E100: Error

def check_security_middleware(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_xframe_options_middleware(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_sts(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_sts_include_subdomains(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_sts_preload(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_content_type_nosniff(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_ssl_redirect(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_secret_key(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_debug(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_xframe_deny(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_allowed_hosts(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Warning]: ...
def check_referrer_policy(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[CheckMessage]: ...
def check_default_hashing_algorithm(app_configs: Sequence[AppConfig] | None, **kwargs: Any) -> Sequence[Error]: ...



from enum import Enum
from typing import List, Type

from django.contrib.auth.models import Group


def create_groups_from_enum(permission_groups: Type[Enum]) -> List[Group]:
    return [
        Group.objects.get_or_create(name=group_name.value)[0]
        for group_name in permission_groups
    ]



import types
from _typeshed import StrPath
from typing import Any, Callable, Dict, Mapping, Optional, Sequence, Tuple, TypeVar, Union

_T = TypeVar("_T")
_localtrace = Callable[[types.FrameType, str, Any], Callable[..., Any]]
_fileModuleFunction = Tuple[str, Optional[str], str]

class CoverageResults:
    def __init__(
        self,
        counts: Optional[Dict[Tuple[str, int], int]] = ...,
        calledfuncs: Optional[Dict[_fileModuleFunction, int]] = ...,
        infile: Optional[StrPath] = ...,
        callers: Optional[Dict[Tuple[_fileModuleFunction, _fileModuleFunction], int]] = ...,
        outfile: Optional[StrPath] = ...,
    ) -> None: ...  # undocumented
    def update(self, other: CoverageResults) -> None: ...
    def write_results(self, show_missing: bool = ..., summary: bool = ..., coverdir: Optional[StrPath] = ...) -> None: ...
    def write_results_file(
        self, path: StrPath, lines: Sequence[str], lnotab: Any, lines_hit: Mapping[int, int], encoding: Optional[str] = ...
    ) -> Tuple[int, int]: ...

class Trace:
    def __init__(
        self,
        count: int = ...,
        trace: int = ...,
        countfuncs: int = ...,
        countcallers: int = ...,
        ignoremods: Sequence[str] = ...,
        ignoredirs: Sequence[str] = ...,
        infile: Optional[StrPath] = ...,
        outfile: Optional[StrPath] = ...,
        timing: bool = ...,
    ) -> None: ...
    def run(self, cmd: Union[str, types.CodeType]) -> None: ...
    def runctx(
        self,
        cmd: Union[str, types.CodeType],
        globals: Optional[Mapping[str, Any]] = ...,
        locals: Optional[Mapping[str, Any]] = ...,
    ) -> None: ...
    def runfunc(self, func: Callable[..., _T], *args: Any, **kw: Any) -> _T: ...
    def file_module_function_of(self, frame: types.FrameType) -> _fileModuleFunction: ...
    def globaltrace_trackcallers(self, frame: types.FrameType, why: str, arg: Any) -> None: ...
    def globaltrace_countfuncs(self, frame: types.FrameType, why: str, arg: Any) -> None: ...
    def globaltrace_lt(self, frame: types.FrameType, why: str, arg: Any) -> None: ...
    def localtrace_trace_and_count(self, frame: types.FrameType, why: str, arg: Any) -> _localtrace: ...
    def localtrace_trace(self, frame: types.FrameType, why: str, arg: Any) -> _localtrace: ...
    def localtrace_count(self, frame: types.FrameType, why: str, arg: Any) -> _localtrace: ...
    def results(self) -> CoverageResults: ...



from datetime import datetime
from typing import IO, Any

from django.core.files.base import File
from django.utils._os import _PathCompatible
from django.utils.functional import LazyObject

class Storage:
    def open(self, name: str, mode: str = ...) -> File: ...
    def save(self, name: str | None, content: IO[Any], max_length: int | None = ...) -> str: ...
    def get_valid_name(self, name: str) -> str: ...
    def get_alternative_name(self, file_root: str, file_ext: str) -> str: ...
    def get_available_name(self, name: str, max_length: int | None = ...) -> str: ...
    def generate_filename(self, filename: _PathCompatible) -> str: ...
    def path(self, name: str) -> str: ...
    def delete(self, name: str) -> None: ...
    def exists(self, name: str) -> bool: ...
    def listdir(self, path: str) -> tuple[list[str], list[str]]: ...
    def size(self, name: str) -> int: ...
    def url(self, name: str | None) -> str: ...
    def get_accessed_time(self, name: str) -> datetime: ...
    def get_created_time(self, name: str) -> datetime: ...
    def get_modified_time(self, name: str) -> datetime: ...

class FileSystemStorage(Storage):
    OS_OPEN_FLAGS: int
    def __init__(
        self,
        location: _PathCompatible | None = ...,
        base_url: str | None = ...,
        file_permissions_mode: int | None = ...,
        directory_permissions_mode: int | None = ...,
    ) -> None: ...
    @property
    def base_location(self) -> _PathCompatible: ...
    @property
    def location(self) -> _PathCompatible: ...
    @property
    def base_url(self) -> str: ...
    @property
    def file_permissions_mode(self) -> int | None: ...
    @property
    def directory_permissions_mode(self) -> int | None: ...

class DefaultStorage(LazyObject): ...

default_storage: Any

def get_storage_class(import_path: str | None = ...) -> type[Storage]: ...



from contextlib import contextmanager
from datetime import datetime
from types import TracebackType
from typing import (
    Any,
    Callable,
    FrozenSet,
    Generic,
    Iterator,
    List,
    Mapping,
    Optional,
    Tuple,
    TypeVar,
)

import kombu
from celery.app.base import Celery
from celery.backends.base import Backend
from celery.utils.graph import DependencyGraph, GraphFormatter
from typing_extensions import Literal
from vine import promise

@contextmanager
def denied_join_result() -> Iterator[None]: ...
@contextmanager
def allow_join_result() -> Iterator[None]: ...

class ResultBase:
    parent: Optional[ResultBase]

_State = Literal["PENDING", "STARTED", "RETRY", "FAILURE", "SUCCESS"]

_R = TypeVar("_R")

class AsyncResult(ResultBase, Generic[_R]):
    app: Celery
    id: str
    backend: Backend
    parent: Optional[ResultBase]
    on_ready: promise
    def __init__(
        self,
        id: str,
        backend: Optional[Backend] = ...,
        task_name: Optional[str] = ...,
        app: Optional[Celery] = ...,
        parent: Optional[ResultBase] = ...,
    ) -> None: ...
    @property
    def ignored(self) -> bool: ...
    @ignored.setter
    def ignored(self, value: bool) -> None: ...
    def then(
        self,
        callback: Callable[..., Any],
        on_error: Optional[Callable[..., Any]] = ...,
        weak: bool = ...,
    ) -> promise: ...
    def as_tuple(
        self,
    ) -> Tuple[int, Optional[Tuple[int, Optional[Any], None]], None]: ...
    def forget(self) -> None: ...
    def revoke(
        self,
        connection: Optional[kombu.Connection] = ...,
        terminate: bool = ...,
        signal: Optional[str] = ...,
        wait: bool = ...,
        timeout: Optional[float] = ...,
    ) -> None: ...
    def get(
        self,
        timeout: Optional[float] = ...,
        propagate: bool = ...,
        interval: float = ...,
        no_ack: bool = ...,
        follow_parents: bool = ...,
        callback: Optional[Callable[..., Any]] = ...,
        on_message: Optional[Callable[..., Any]] = ...,
        on_interval: Optional[Callable[..., Any]] = ...,
        disable_sync_subtasks: bool = ...,
        EXCEPTION_STATES: FrozenSet[str] = ...,
        PROPAGATE_STATES: FrozenSet[str] = ...,
    ) -> _R: ...
    def collect(
        self, intermediate: bool = ..., **kwargs: Any
    ) -> Iterator[Tuple[AsyncResult[Any], object]]: ...
    def get_leaf(self) -> object: ...
    def iterdeps(
        self, intermediate: bool = ...
    ) -> Iterator[Tuple[Optional[AsyncResult[Any]], AsyncResult[Any]]]: ...
    def ready(self) -> bool: ...
    def successful(self) -> bool: ...
    def failed(self) -> bool: ...
    def throw(self, *args: Any, **kwargs: Any) -> None: ...
    def maybe_throw(
        self, propagate: bool = ..., callback: Optional[Callable[..., Any]] = ...
    ) -> object: ...
    def build_graph(
        self, intermediate: bool = ..., formatter: Optional[GraphFormatter] = ...
    ) -> DependencyGraph: ...
    @property
    def graph(self) -> DependencyGraph: ...
    @property
    def supports_native_join(self) -> bool: ...
    @property
    def children(
        self,
    ) -> Optional[
        List[Tuple[int, Optional[Tuple[int, Optional[Any], None]], None]]
    ]: ...
    @property
    def result(self) -> _R | BaseException: ...
    @property
    def info(self) -> Any: ...
    @property
    def traceback(self) -> Optional[TracebackType]: ...
    @property
    def state(self) -> _State: ...
    @property
    def status(
        self,
    ) -> _State: ...
    @property
    def task_id(self) -> str: ...
    @task_id.setter
    def task_id(self, id: str) -> None: ...
    @property
    def name(self) -> Optional[str]: ...
    @property
    def args(self) -> Optional[Tuple[Any, ...]]: ...
    @property
    def kwargs(self) -> Optional[Mapping[str, Any]]: ...
    @property
    def worker(self) -> Optional[str]: ...
    @property
    def date_done(self) -> Optional[datetime]: ...
    @property
    def retries(self) -> Optional[int]: ...
    @property
    def queue(self) -> Optional[str]: ...

class EagerResult(AsyncResult[_R]): ...
class ResultSet(ResultBase): ...
class GroupResult(ResultSet): ...



from collections.abc import Sequence
from typing import Any, Protocol, Union  # noqa: Y037  # https://github.com/python/mypy/issues/12392

from django.db.models import Model, QuerySet
from rest_framework.request import Request
from rest_framework.views import APIView
from typing_extensions import TypeAlias

SAFE_METHODS: Sequence[str]

class _SupportsHasPermission(Protocol):
    def has_permission(self, request: Request, view: APIView) -> bool: ...
    def has_object_permission(self, request: Request, view: APIView, obj: Any) -> bool: ...

# https://github.com/python/mypy/issues/12392
_PermissionClass: TypeAlias = Union[type[BasePermission], OperandHolder, SingleOperandHolder]

class OperationHolderMixin:
    def __and__(self, other: _PermissionClass) -> OperandHolder: ...
    def __or__(self, other: _PermissionClass) -> OperandHolder: ...
    def __rand__(self, other: _PermissionClass) -> OperandHolder: ...
    def __ror__(self, other: _PermissionClass) -> OperandHolder: ...
    def __invert__(self) -> SingleOperandHolder: ...

class SingleOperandHolder(OperationHolderMixin):
    operator_class: _SupportsHasPermission
    op1_class: _PermissionClass
    def __init__(self, operator_class: _SupportsHasPermission, op1_class: _PermissionClass): ...
    def __call__(self, *args, **kwargs) -> _SupportsHasPermission: ...

class OperandHolder(OperationHolderMixin):
    operator_class: _SupportsHasPermission
    op1_class: _PermissionClass
    op2_class: _PermissionClass
    def __init__(
        self, operator_class: _SupportsHasPermission, op1_class: _PermissionClass, op2_class: _PermissionClass
    ): ...
    def __call__(self, *args, **kwargs) -> _SupportsHasPermission: ...

class AND(_SupportsHasPermission):
    def __init__(self, op1: _SupportsHasPermission, op2: _SupportsHasPermission) -> None: ...

class OR(_SupportsHasPermission):
    def __init__(self, op1: _SupportsHasPermission, op2: _SupportsHasPermission) -> None: ...

class NOT(_SupportsHasPermission):
    def __init__(self, op1: _SupportsHasPermission) -> None: ...

class BasePermissionMetaclass(OperationHolderMixin, type): ...  # type: ignore[misc]

class BasePermission(metaclass=BasePermissionMetaclass):  # type: ignore[misc]
    def has_permission(self, request: Request, view: APIView) -> bool: ...
    def has_object_permission(self, request: Request, view: APIView, obj: Any) -> bool: ...

class AllowAny(BasePermission): ...
class IsAuthenticated(BasePermission): ...
class IsAdminUser(BasePermission): ...
class IsAuthenticatedOrReadOnly(BasePermission): ...

class DjangoModelPermissions(BasePermission):
    perms_map: dict[str, list[str]]
    authenticated_users_only: bool
    def get_required_permissions(self, method: str, model_cls: type[Model]) -> list[str]: ...
    def _queryset(self, view: APIView) -> QuerySet: ...

class DjangoModelPermissionsOrAnonReadOnly(DjangoModelPermissions): ...

class DjangoObjectPermissions(DjangoModelPermissions):
    def get_required_object_permissions(self, method: str, model_cls: type[Model]) -> list[str]: ...



import threading
import unittest
from collections.abc import Callable, Collection, Generator, Iterable, Iterator, Mapping, Sequence
from contextlib import contextmanager
from types import TracebackType
from typing import Any, overload

from _typeshed import Self
from django.core.exceptions import ImproperlyConfigured
from django.core.handlers.wsgi import WSGIHandler
from django.core.servers.basehttp import ThreadedWSGIServer, WSGIRequestHandler
from django.db import connections  # noqa: F401
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.base import Model
from django.db.models.query import QuerySet, RawQuerySet
from django.forms import BaseFormSet, Form
from django.forms.fields import EmailField
from django.http import HttpRequest
from django.http.response import FileResponse, HttpResponseBase
from django.template.base import Template
from django.test.client import AsyncClient, Client
from django.test.html import Element
from django.test.utils import CaptureQueriesContext, ContextList
from django.utils.functional import classproperty

def to_list(value: Any) -> list[Any]: ...
def assert_and_parse_html(self: Any, html: str, user_msg: str, msg: str) -> Element: ...

class _AssertNumQueriesContext(CaptureQueriesContext):
    test_case: SimpleTestCase
    num: int
    def __init__(self, test_case: Any, num: Any, connection: BaseDatabaseWrapper) -> None: ...

class _AssertTemplateUsedContext:
    test_case: SimpleTestCase
    template_name: str
    rendered_templates: list[Template]
    rendered_template_names: list[str]
    context: ContextList
    def __init__(self, test_case: Any, template_name: Any) -> None: ...
    def on_template_render(self, sender: Any, signal: Any, template: Any, context: Any, **kwargs: Any) -> None: ...
    def test(self) -> None: ...
    def message(self) -> str: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

class _AssertTemplateNotUsedContext(_AssertTemplateUsedContext): ...

class _DatabaseFailure:
    wrapped: Any
    message: str
    def __init__(self, wrapped: Any, message: str) -> None: ...
    def __call__(self) -> None: ...

class SimpleTestCase(unittest.TestCase):
    client_class: type[Client]
    client: Client
    async_client_class: type[AsyncClient]
    async_client: AsyncClient
    allow_database_queries: bool
    # TODO: str -> Literal['__all__']
    databases: set[str] | str
    def __call__(self, result: unittest.TestResult | None = ...) -> None: ...
    def settings(self, **kwargs: Any) -> Any: ...
    def modify_settings(self, **kwargs: Any) -> Any: ...
    def assertRedirects(
        self,
        response: HttpResponseBase,
        expected_url: str,
        status_code: int = ...,
        target_status_code: int = ...,
        msg_prefix: str = ...,
        fetch_redirect_response: bool = ...,
    ) -> None: ...
    def assertURLEqual(
        self,
        url1: str | Any,  # Any for reverse_lazy() support
        url2: str | Any,
        msg_prefix: str = ...,
    ) -> None: ...
    def assertContains(
        self,
        response: HttpResponseBase,
        text: bytes | int | str,
        count: int | None = ...,
        status_code: int = ...,
        msg_prefix: str = ...,
        html: bool = ...,
    ) -> None: ...
    def assertNotContains(
        self,
        response: HttpResponseBase,
        text: bytes | str,
        status_code: int = ...,
        msg_prefix: str = ...,
        html: bool = ...,
    ) -> None: ...
    @overload
    def assertFormError(
        self,
        form: Form,
        field: str | None,
        errors: list[str] | str,
        msg_prefix: str = ...,
    ) -> None: ...
    @overload
    def assertFormError(  # old signature, deprecated in Django 4.1
        self,
        response: HttpResponseBase,
        form: str,
        field: str | None,
        errors: list[str] | str,
        msg_prefix: str = ...,
    ) -> None: ...
    # assertFormsetError (lowercase "set") deprecated in Django 4.2
    @overload
    def assertFormsetError(
        self,
        formset: BaseFormSet,
        form_index: int | None,
        field: str | None,
        errors: list[str] | str,
        msg_prefix: str = ...,
    ) -> None: ...
    @overload
    def assertFormsetError(  # old signature, deprecated in Django 4.1
        self,
        response: HttpResponseBase,
        formset: str,
        form_index: int | None,
        field: str | None,
        errors: list[str] | str,
        msg_prefix: str = ...,
    ) -> None: ...
    @overload
    def assertFormSetError(
        self,
        formset: BaseFormSet,
        form_index: int | None,
        field: str | None,
        errors: list[str] | str,
        msg_prefix: str = ...,
    ) -> None: ...
    @overload
    def assertFormSetError(
        self,
        response: HttpResponseBase,
        formset: str,
        form_index: int | None,
        field: str | None,
        errors: list[str] | str,
        msg_prefix: str = ...,
    ) -> None: ...
    def assertTemplateUsed(
        self,
        response: HttpResponseBase | str | None = ...,
        template_name: str | None = ...,
        msg_prefix: str = ...,
        count: int | None = ...,
    ) -> _AssertTemplateUsedContext | None: ...
    def assertTemplateNotUsed(
        self, response: HttpResponseBase | str = ..., template_name: str | None = ..., msg_prefix: str = ...
    ) -> _AssertTemplateNotUsedContext | None: ...
    def assertRaisesMessage(
        self, expected_exception: type[Exception], expected_message: str, *args: Any, **kwargs: Any
    ) -> Any: ...
    def assertWarnsMessage(
        self, expected_warning: type[Exception], expected_message: str, *args: Any, **kwargs: Any
    ) -> Any: ...
    def assertFieldOutput(
        self,
        fieldclass: type[EmailField],
        valid: dict[str, str],
        invalid: dict[str, list[str]],
        field_args: Iterable[Any] | None = ...,
        field_kwargs: Mapping[str, Any] | None = ...,
        empty_value: str = ...,
    ) -> Any: ...
    def assertHTMLEqual(self, html1: str, html2: str, msg: str | None = ...) -> None: ...
    def assertHTMLNotEqual(self, html1: str, html2: str, msg: str | None = ...) -> None: ...
    def assertInHTML(self, needle: str, haystack: str, count: int | None = ..., msg_prefix: str = ...) -> None: ...
    def assertJSONEqual(
        self,
        raw: str,
        expected_data: dict[str, Any] | list[Any] | str | int | float | bool | None,
        msg: str | None = ...,
    ) -> None: ...
    def assertJSONNotEqual(
        self,
        raw: str,
        expected_data: dict[str, Any] | list[Any] | str | int | float | bool | None,
        msg: str | None = ...,
    ) -> None: ...
    def assertXMLEqual(self, xml1: str, xml2: str, msg: str | None = ...) -> None: ...
    def assertXMLNotEqual(self, xml1: str, xml2: str, msg: str | None = ...) -> None: ...

class TransactionTestCase(SimpleTestCase):
    reset_sequences: bool
    available_apps: Any
    fixtures: Any
    multi_db: bool
    serialized_rollback: bool
    # assertQuerysetEqual (lowercase "set") deprecated in Django 4.2
    def assertQuerysetEqual(
        self,
        qs: Iterator[Any] | list[Model] | QuerySet | RawQuerySet,
        values: Collection[Any],
        transform: Callable[[Model], Any] | type[str] = ...,
        ordered: bool = ...,
        msg: str | None = ...,
    ) -> None: ...
    def assertQuerySetEqual(
        self,
        qs: Iterator[Any] | list[Model] | QuerySet | RawQuerySet,
        values: Collection[Any],
        transform: Callable[[Model], Any] | type[str] = ...,
        ordered: bool = ...,
        msg: str | None = ...,
    ) -> None: ...
    @overload
    def assertNumQueries(self, num: int, using: str = ...) -> _AssertNumQueriesContext: ...  # type: ignore
    @overload
    def assertNumQueries(
        self, num: int, func: Callable[..., Any], *args: Any, using: str = ..., **kwargs: Any
    ) -> None: ...

class TestCase(TransactionTestCase):
    @classmethod
    def setUpTestData(cls) -> None: ...
    @classmethod
    @contextmanager
    def captureOnCommitCallbacks(
        cls, *, using: str = ..., execute: bool = ...
    ) -> Generator[list[Callable[[], Any]], None, None]: ...

class CheckCondition:
    conditions: Sequence[tuple[Callable, str]]
    def __init__(self, *conditions: tuple[Callable, str]) -> None: ...
    def add_condition(self, condition: Callable, reason: str) -> CheckCondition: ...
    def __get__(self, instance: None, cls: type[TransactionTestCase] | None = ...) -> bool: ...

def skipIfDBFeature(*features: Any) -> Callable: ...
def skipUnlessDBFeature(*features: Any) -> Callable: ...
def skipUnlessAnyDBFeature(*features: Any) -> Callable: ...

class QuietWSGIRequestHandler(WSGIRequestHandler): ...

class FSFilesHandler(WSGIHandler):
    application: Any
    base_url: Any
    def __init__(self, application: Any) -> None: ...
    def file_path(self, url: Any) -> str: ...
    def serve(self, request: HttpRequest) -> FileResponse: ...

class _StaticFilesHandler(FSFilesHandler):
    def get_base_dir(self) -> str: ...
    def get_base_url(self) -> str: ...

class _MediaFilesHandler(FSFilesHandler):
    def get_base_dir(self) -> str: ...
    def get_base_url(self) -> str: ...

class LiveServerThread(threading.Thread):
    host: str
    port: int
    is_ready: threading.Event
    error: ImproperlyConfigured | None
    static_handler: type[WSGIHandler]
    connections_override: dict[str, BaseDatabaseWrapper]
    def __init__(
        self,
        host: str,
        static_handler: type[WSGIHandler],
        connections_override: dict[str, BaseDatabaseWrapper] = ...,
        port: int = ...,
    ) -> None: ...
    httpd: ThreadedWSGIServer
    def terminate(self) -> None: ...

class LiveServerTestCase(TransactionTestCase):
    host: str
    port: int
    server_thread_class: type[Any]
    server_thread: Any
    static_handler: Any
    @classproperty
    def live_server_url(cls: Any) -> str: ...
    @classproperty
    def allowed_host(cls: Any) -> str: ...

class SerializeMixin:
    lockfile: Any
    @classmethod
    def setUpClass(cls: type[Self]) -> None: ...
    @classmethod
    def tearDownClass(cls: type[Self]) -> None: ...

def connections_support_transactions(aliases: Iterable[str] | None = ...) -> bool: ...



from typing import IO, Any, List, Optional, Text, TypeVar, Union

def split(s: Optional[str], comments: bool = ..., posix: bool = ...) -> List[str]: ...

_SLT = TypeVar("_SLT", bound=shlex)

class shlex:
    def __init__(self, instream: Union[IO[Any], Text] = ..., infile: IO[Any] = ..., posix: bool = ...) -> None: ...
    def __iter__(self: _SLT) -> _SLT: ...
    def next(self) -> str: ...
    def get_token(self) -> Optional[str]: ...
    def push_token(self, _str: str) -> None: ...
    def read_token(self) -> str: ...
    def sourcehook(self, filename: str) -> None: ...
    def push_source(self, stream: IO[Any], filename: str = ...) -> None: ...
    def pop_source(self) -> IO[Any]: ...
    def error_leader(self, file: str = ..., line: int = ...) -> str: ...
    commenters: str
    wordchars: str
    whitespace: str
    escape: str
    quotes: str
    escapedquotes: str
    whitespace_split: bool
    infile: IO[Any]
    source: Optional[str]
    debug: int
    lineno: int
    token: Any
    eof: Optional[str]



import decimal
import uuid
from collections.abc import Callable, Iterable, Sequence
from datetime import date
from datetime import datetime as real_datetime
from datetime import time, timedelta
from typing import Any, Generic, Protocol, TypeVar, overload

from _typeshed import Self
from django.core import validators  # due to weird mypy.stubtest error
from django.core.checks import CheckMessage
from django.core.exceptions import FieldDoesNotExist as FieldDoesNotExist
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Model
from django.db.models.expressions import Col, Combinable
from django.db.models.fields.reverse_related import ForeignObjectRel
from django.db.models.query_utils import Q, RegisterLookupMixin
from django.forms import Field as FormField
from django.forms import Widget
from django.utils.datastructures import DictWrapper
from django.utils.functional import _Getter, _StrOrPromise
from typing_extensions import TypeAlias

class Empty: ...
class NOT_PROVIDED: ...

BLANK_CHOICE_DASH: list[tuple[str, str]]

_Choice: TypeAlias = tuple[Any, Any]

_ChoiceNamedGroup: TypeAlias = tuple[str, Iterable[_Choice]]
_FieldChoices: TypeAlias = Iterable[_Choice | _ChoiceNamedGroup]
_ChoicesList: TypeAlias = Sequence[_Choice] | Sequence[_ChoiceNamedGroup]
_LimitChoicesTo: TypeAlias = Q | dict[str, Any]

class _ChoicesCallable(Protocol):
    def __call__(self) -> _FieldChoices: ...

_AllLimitChoicesTo: TypeAlias = _LimitChoicesTo | _ChoicesCallable  # noqa: Y047
_ErrorMessagesT: TypeAlias = dict[str, Any]

# __set__ value type
_ST = TypeVar("_ST", contravariant=True)
# __get__ return type
_GT = TypeVar("_GT", covariant=True)

class Field(RegisterLookupMixin, Generic[_ST, _GT]):
    """
    Typing model fields.

    How does this work?
    Let's take a look at the self-contained example
    (it is way easier than our django implementation, but has the same concept).

    To understand this example you need:
    1. Be familiar with descriptors: https://docs.python.org/3/howto/descriptor.html
    2. Follow our explanation bellow

    Let's start with defining our fake model class and fake integer field.

    .. code:: python

        from typing import Generic, Union

        class Model(object):
            ...

        _SetType = Union[int, float]  # You can assign ints and floats
        _GetType = int  # access type is always `int`

        class IntField(object):
            def __get__(self, instance: Model, owner) -> _GetType:
                ...

            def __set__(self, instance, value: _SetType) -> None:
                ...

    Now, let's create our own example model,
    this would be something like ``User`` in your own apps:

    .. code:: python

        class Example(Model):
            count = IntField()

    And now, lets test that our reveal type works:

    .. code:: python

        example = Example()
        reveal_type(example.count)
        # Revealed type is "builtins.int"

        example.count = 1.5  # ok
        example.count = 'a'
        # Incompatible types in assignment
        # (expression has type "str", variable has type "Union[int, float]")

    Notice, that this is not magic. This is how descriptors work with ``mypy``.

    We also need ``_pyi_private_set_type`` attributes
    and friends to help inside our plugin.
    It is required to enhance parts like ``filter`` queries.
    """

    _pyi_private_set_type: Any
    _pyi_private_get_type: Any
    _pyi_lookup_exact_type: Any

    widget: Widget
    help_text: _StrOrPromise
    attname: str
    auto_created: bool
    primary_key: bool
    remote_field: ForeignObjectRel | None
    is_relation: bool
    related_model: type[Model] | None
    one_to_many: bool | None
    one_to_one: bool | None
    many_to_many: bool | None
    many_to_one: bool | None
    max_length: int | None
    model: type[Model]
    name: str
    verbose_name: _StrOrPromise
    description: str | _Getter[str]
    blank: bool
    null: bool
    unique: bool
    editable: bool
    empty_strings_allowed: bool
    choices: _ChoicesList | None
    db_column: str | None
    column: str
    concrete: bool
    default: Any
    error_messages: _ErrorMessagesT
    empty_values: Sequence[Any]
    creation_counter: int
    auto_creation_counter: int
    default_validators: Sequence[validators._ValidatorCallable]
    default_error_messages: dict[str, str]
    hidden: bool
    system_check_removed_details: Any | None
    system_check_deprecated_details: Any | None
    non_db_attrs: tuple[str, ...]
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        primary_key: bool = ...,
        max_length: int | None = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        rel: ForeignObjectRel | None = ...,
        default: Any = ...,
        editable: bool = ...,
        serialize: bool = ...,
        unique_for_date: str | None = ...,
        unique_for_month: str | None = ...,
        unique_for_year: str | None = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        auto_created: bool = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...
    def __set__(self, instance: Any, value: _ST) -> None: ...
    # class access
    @overload
    def __get__(self: Self, instance: None, owner: Any) -> Self: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner: Any) -> _GT: ...
    # non-Model instances
    @overload
    def __get__(self: Self, instance: Any, owner: Any) -> Self: ...
    def deconstruct(self) -> Any: ...
    def set_attributes_from_name(self, name: str) -> None: ...
    def db_type_parameters(self, connection: BaseDatabaseWrapper) -> DictWrapper: ...
    def db_check(self, connection: BaseDatabaseWrapper) -> str | None: ...
    def db_type(self, connection: BaseDatabaseWrapper) -> str | None: ...
    def db_parameters(self, connection: BaseDatabaseWrapper) -> dict[str, str | None]: ...
    def pre_save(self, model_instance: Model, add: bool) -> Any: ...
    def get_prep_value(self, value: Any) -> Any: ...
    def get_db_prep_value(self, value: Any, connection: BaseDatabaseWrapper, prepared: bool = ...) -> Any: ...
    def get_db_prep_save(self, value: Any, connection: BaseDatabaseWrapper) -> Any: ...
    def get_internal_type(self) -> str: ...
    # TODO: plugin support
    def formfield(self, form_class: Any | None = ..., choices_form_class: Any | None = ..., **kwargs: Any) -> Any: ...
    def save_form_data(self, instance: Model, data: Any) -> None: ...
    def contribute_to_class(self, cls: type[Model], name: str, private_only: bool = ...) -> None: ...
    def to_python(self, value: Any) -> Any: ...
    @property
    def validators(self) -> list[validators._ValidatorCallable]: ...
    def run_validators(self, value: Any) -> None: ...
    def validate(self, value: Any, model_instance: Model | None) -> None: ...
    def clean(self, value: Any, model_instance: Model | None) -> Any: ...
    def get_choices(
        self,
        include_blank: bool = ...,
        blank_choice: _ChoicesList = ...,
        limit_choices_to: _LimitChoicesTo | None = ...,
        ordering: Sequence[str] = ...,
    ) -> _ChoicesList: ...
    def _get_flatchoices(self) -> list[_Choice]: ...
    @property
    def flatchoices(self) -> list[_Choice]: ...
    def has_default(self) -> bool: ...
    def get_default(self) -> Any: ...
    def check(self, **kwargs: Any) -> list[CheckMessage]: ...
    def get_col(self, alias: str, output_field: Field | None = ...) -> Col: ...
    @property
    def cached_col(self) -> Col: ...
    def value_from_object(self, obj: Model) -> _GT: ...
    def get_attname(self) -> str: ...
    def value_to_string(self, obj: Model) -> str: ...

class IntegerField(Field[_ST, _GT]):
    _pyi_private_set_type: float | int | str | Combinable
    _pyi_private_get_type: int
    _pyi_lookup_exact_type: str | int

class PositiveIntegerRelDbTypeMixin:
    def rel_db_type(self, connection: BaseDatabaseWrapper) -> str: ...

class PositiveIntegerField(PositiveIntegerRelDbTypeMixin, IntegerField[_ST, _GT]): ...
class PositiveSmallIntegerField(PositiveIntegerRelDbTypeMixin, SmallIntegerField[_ST, _GT]): ...
class PositiveBigIntegerField(PositiveIntegerRelDbTypeMixin, BigIntegerField[_ST, _GT]): ...
class SmallIntegerField(IntegerField[_ST, _GT]): ...
class BigIntegerField(IntegerField[_ST, _GT]): ...

class FloatField(Field[_ST, _GT]):
    _pyi_private_set_type: float | int | str | Combinable
    _pyi_private_get_type: float
    _pyi_lookup_exact_type: float

class DecimalField(Field[_ST, _GT]):
    _pyi_private_set_type: str | float | decimal.Decimal | Combinable
    _pyi_private_get_type: decimal.Decimal
    _pyi_lookup_exact_type: str | decimal.Decimal
    # attributes
    max_digits: int
    decimal_places: int
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        max_digits: int | None = ...,
        decimal_places: int | None = ...,
        *,
        primary_key: bool = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...

class CharField(Field[_ST, _GT]):
    _pyi_private_set_type: str | int | Combinable
    _pyi_private_get_type: str
    # objects are converted to string before comparison
    _pyi_lookup_exact_type: Any
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        primary_key: bool = ...,
        max_length: int | None = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: str | None = ...,
        unique_for_month: str | None = ...,
        unique_for_year: str | None = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
        *,
        db_collation: str | None = ...,
    ) -> None: ...

class CommaSeparatedIntegerField(CharField[_ST, _GT]): ...

class SlugField(CharField[_ST, _GT]):
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        primary_key: bool = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: str | None = ...,
        unique_for_month: str | None = ...,
        unique_for_year: str | None = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
        *,
        max_length: int | None = ...,
        db_index: bool = ...,
        allow_unicode: bool = ...,
    ) -> None: ...

class EmailField(CharField[_ST, _GT]): ...

class URLField(CharField[_ST, _GT]):
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        *,
        primary_key: bool = ...,
        max_length: int | None = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        rel: ForeignObjectRel | None = ...,
        default: Any = ...,
        editable: bool = ...,
        serialize: bool = ...,
        unique_for_date: str | None = ...,
        unique_for_month: str | None = ...,
        unique_for_year: str | None = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        auto_created: bool = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...

class TextField(Field[_ST, _GT]):
    _pyi_private_set_type: str | Combinable
    _pyi_private_get_type: str
    # objects are converted to string before comparison
    _pyi_lookup_exact_type: Any
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        primary_key: bool = ...,
        max_length: int | None = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: str | None = ...,
        unique_for_month: str | None = ...,
        unique_for_year: str | None = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
        *,
        db_collation: str | None = ...,
    ) -> None: ...

class BooleanField(Field[_ST, _GT]):
    _pyi_private_set_type: bool | Combinable
    _pyi_private_get_type: bool
    _pyi_lookup_exact_type: bool

class NullBooleanField(BooleanField[_ST, _GT]):
    _pyi_private_set_type: bool | Combinable | None  # type: ignore
    _pyi_private_get_type: bool | None  # type: ignore
    _pyi_lookup_exact_type: bool | None  # type: ignore

class IPAddressField(Field[_ST, _GT]):
    _pyi_private_set_type: str | Combinable
    _pyi_private_get_type: str

class GenericIPAddressField(Field[_ST, _GT]):
    _pyi_private_set_type: str | int | Callable[..., Any] | Combinable
    _pyi_private_get_type: str

    default_error_messages: dict[str, str]
    unpack_ipv4: bool
    protocol: str
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: Any | None = ...,
        protocol: str = ...,
        unpack_ipv4: bool = ...,
        primary_key: bool = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...

class DateTimeCheckMixin: ...

class DateField(DateTimeCheckMixin, Field[_ST, _GT]):
    _pyi_private_set_type: str | date | Combinable
    _pyi_private_get_type: date
    _pyi_lookup_exact_type: str | date
    auto_now: bool
    auto_now_add: bool
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        auto_now: bool = ...,
        auto_now_add: bool = ...,
        *,
        primary_key: bool = ...,
        max_length: int | None = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...

class TimeField(DateTimeCheckMixin, Field[_ST, _GT]):
    _pyi_private_set_type: str | time | real_datetime | Combinable
    _pyi_private_get_type: time
    auto_now: bool
    auto_now_add: bool
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        auto_now: bool = ...,
        auto_now_add: bool = ...,
        *,
        primary_key: bool = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...

class DateTimeField(DateField[_ST, _GT]):
    _pyi_private_set_type: str | real_datetime | date | Combinable
    _pyi_private_get_type: real_datetime
    _pyi_lookup_exact_type: str | real_datetime

class UUIDField(Field[_ST, _GT]):
    _pyi_private_set_type: str | uuid.UUID
    _pyi_private_get_type: uuid.UUID
    _pyi_lookup_exact_type: uuid.UUID | str
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        *,
        name: str | None = ...,
        primary_key: bool = ...,
        max_length: int | None = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        rel: ForeignObjectRel | None = ...,
        default: Any = ...,
        editable: bool = ...,
        serialize: bool = ...,
        unique_for_date: str | None = ...,
        unique_for_month: str | None = ...,
        unique_for_year: str | None = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        auto_created: bool = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...

class FilePathField(Field[_ST, _GT]):
    path: Any
    match: str | None
    recursive: bool
    allow_files: bool
    allow_folders: bool
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = ...,
        name: str | None = ...,
        path: str | Callable[..., str] = ...,
        match: str | None = ...,
        recursive: bool = ...,
        allow_files: bool = ...,
        allow_folders: bool = ...,
        *,
        primary_key: bool = ...,
        max_length: int = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        choices: _FieldChoices | None = ...,
        help_text: _StrOrPromise = ...,
        db_column: str | None = ...,
        db_tablespace: str | None = ...,
        validators: Iterable[validators._ValidatorCallable] = ...,
        error_messages: _ErrorMessagesT | None = ...,
    ) -> None: ...

class BinaryField(Field[_ST, _GT]):
    _pyi_private_get_type: bytes | memoryview

class DurationField(Field[_ST, _GT]):
    _pyi_private_get_type: timedelta

class AutoFieldMixin:
    db_returning: bool

class AutoFieldMeta(type): ...

class AutoField(AutoFieldMixin, IntegerField[_ST, _GT], metaclass=AutoFieldMeta):
    _pyi_private_set_type: Combinable | int | str
    _pyi_private_get_type: int
    _pyi_lookup_exact_type: str | int

class BigAutoField(AutoFieldMixin, BigIntegerField[_ST, _GT]): ...
class SmallAutoField(AutoFieldMixin, SmallIntegerField[_ST, _GT]): ...



from typing import Any

from django.db import models
from django.db.models.lookups import PostgresOperatorLookup
from psycopg2.extras import DateRange, DateTimeTZRange, NumericRange, Range
from typing_extensions import Literal

class RangeBoundary(models.Expression):
    lower: str
    upper: str
    def __init__(self, inclusive_lower: bool = ..., inclusive_upper: bool = ...) -> None: ...

class RangeOperators:
    EQUAL: Literal["="]
    NOT_EQUAL: Literal["<>"]
    CONTAINS: Literal["@>"]
    CONTAINED_BY: Literal["<@"]
    OVERLAPS: Literal["&&"]
    FULLY_LT: Literal["<<"]
    FULLY_GT: Literal[">>"]
    NOT_LT: Literal["&>"]
    NOT_GT: Literal["&<"]
    ADJACENT_TO: Literal["-|-"]

class RangeField(models.Field):
    empty_strings_allowed: bool
    base_field: models.Field
    range_type: type[Range]
    def get_prep_value(self, value: Any) -> Any | None: ...
    def to_python(self, value: Any) -> Any: ...

class IntegerRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> NumericRange: ...

class BigIntegerRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> NumericRange: ...

class DecimalRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> NumericRange: ...

class DateTimeRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> DateTimeTZRange: ...

class DateRangeField(RangeField):
    def __get__(self, instance: Any, owner: Any) -> DateRange: ...

class DateTimeRangeContains(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class RangeContainedBy(PostgresOperatorLookup):
    lookup_name: str
    type_mapping: dict[str, str]
    postgres_operator: str

class FullyLessThan(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class FullGreaterThan(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class NotLessThan(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class NotGreaterThan(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class AdjacentToLookup(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class RangeStartsWith(models.Transform):
    lookup_name: str
    function: str
    @property
    def output_field(self) -> models.Field: ...

class RangeEndsWith(models.Transform):
    lookup_name: str
    function: str
    @property
    def output_field(self) -> models.Field: ...

class IsEmpty(models.Transform):
    lookup_name: str
    function: str
    output_field: models.BooleanField

class LowerInclusive(models.Transform):
    lookup_name: str
    function: str
    output_field: models.BooleanField

class LowerInfinite(models.Transform):
    lookup_name: str
    function: str
    output_field: models.BooleanField

class UpperInclusive(models.Transform):
    lookup_name: str
    function: str
    output_field: models.BooleanField

class UpperInfinite(models.Transform):
    lookup_name: str
    function: str
    output_field: models.BooleanField



import sys
from typing import BinaryIO, Optional, Text, Union

_File = Union[Text, BinaryIO]

class Error(Exception): ...

if sys.version_info >= (3, 7):
    def encode(
        in_file: _File, out_file: _File, name: Optional[str] = ..., mode: Optional[int] = ..., *, backtick: bool = ...
    ) -> None: ...

else:
    def encode(in_file: _File, out_file: _File, name: Optional[str] = ..., mode: Optional[int] = ...) -> None: ...

def decode(in_file: _File, out_file: Optional[_File] = ..., mode: Optional[int] = ..., quiet: int = ...) -> None: ...



from django.contrib.flatpages.models import FlatPage
from django.http.request import HttpRequest
from django.http.response import HttpResponse

DEFAULT_TEMPLATE: str

def flatpage(request: HttpRequest, url: str) -> HttpResponse: ...
def render_flatpage(request: HttpRequest, f: FlatPage) -> HttpResponse: ...



from collections.abc import Iterable, Mapping, Sequence
from typing import Any

from django import forms
from django.contrib.admin.sites import AdminSite
from django.core.files.base import File
from django.db.models.fields import _FieldChoices
from django.db.models.fields.reverse_related import ForeignObjectRel, ManyToManyRel, ManyToOneRel
from django.forms.models import ModelChoiceIterator
from django.forms.widgets import _OptAttrs
from django.utils.functional import _StrOrPromise

class FilteredSelectMultiple(forms.SelectMultiple):
    verbose_name: _StrOrPromise
    is_stacked: bool
    def __init__(
        self,
        verbose_name: _StrOrPromise,
        is_stacked: bool,
        attrs: _OptAttrs | None = ...,
        choices: _FieldChoices = ...,
    ) -> None: ...

class AdminDateWidget(forms.DateInput):
    def __init__(self, attrs: _OptAttrs | None = ..., format: str | None = ...) -> None: ...

class AdminTimeWidget(forms.TimeInput):
    def __init__(self, attrs: _OptAttrs | None = ..., format: str | None = ...) -> None: ...

class AdminSplitDateTime(forms.SplitDateTimeWidget):
    template_name: str
    def __init__(self, attrs: _OptAttrs | None = ...) -> None: ...
    def get_context(self, name: str, value: Any, attrs: _OptAttrs | None) -> dict[str, Any]: ...

class AdminRadioSelect(forms.RadioSelect): ...
class AdminFileWidget(forms.ClearableFileInput): ...

def url_params_from_lookup_dict(lookups: Any) -> dict[str, str]: ...

class ForeignKeyRawIdWidget(forms.TextInput):
    rel: ManyToOneRel
    admin_site: AdminSite
    db: str | None
    def __init__(
        self, rel: ManyToOneRel, admin_site: AdminSite, attrs: _OptAttrs | None = ..., using: str | None = ...
    ) -> None: ...
    def base_url_parameters(self) -> dict[str, str]: ...
    def get_context(self, name: str, value: Any, attrs: _OptAttrs | None) -> dict[str, Any]: ...
    def url_parameters(self) -> dict[str, str]: ...
    def label_and_url_for_value(self, value: Any) -> tuple[str, str]: ...

class ManyToManyRawIdWidget(ForeignKeyRawIdWidget):
    rel: ManyToManyRel  # type: ignore
    def get_context(self, name: str, value: Any, attrs: _OptAttrs | None) -> dict[str, Any]: ...
    def url_parameters(self) -> dict[str, str]: ...
    def label_and_url_for_value(self, value: Any) -> tuple[str, str]: ...
    def format_value(self, value: Any) -> str | None: ...
    def value_from_datadict(self, data: Mapping[str, Any], files: Mapping[str, Iterable[File]], name: str) -> Any: ...

class RelatedFieldWidgetWrapper(forms.Widget):
    template_name: str
    choices: ModelChoiceIterator
    widget: forms.ChoiceWidget
    rel: ManyToOneRel
    can_add_related: bool
    can_change_related: bool
    can_delete_related: bool
    can_view_related: bool
    admin_site: AdminSite
    def __init__(
        self,
        widget: forms.ChoiceWidget,
        rel: ManyToOneRel,
        admin_site: AdminSite,
        can_add_related: bool | None = ...,
        can_change_related: bool = ...,
        can_delete_related: bool = ...,
        can_view_related: bool = ...,
    ) -> None: ...
    @property
    def is_hidden(self) -> bool: ...
    def get_related_url(self, info: tuple[str, str], action: str, *args: Any) -> str: ...
    def get_context(self, name: str, value: Any, attrs: _OptAttrs | None) -> dict[str, Any]: ...
    def value_from_datadict(self, data: Mapping[str, Any], files: Mapping[str, Iterable[File]], name: str) -> Any: ...
    def value_omitted_from_data(
        self, data: Mapping[str, Any], files: Mapping[str, Iterable[File]], name: str
    ) -> bool: ...
    def id_for_label(self, id_: str) -> str: ...

class AdminTextareaWidget(forms.Textarea):
    def __init__(self, attrs: _OptAttrs | None = ...) -> None: ...

class AdminTextInputWidget(forms.TextInput):
    def __init__(self, attrs: _OptAttrs | None = ...) -> None: ...

class AdminEmailInputWidget(forms.EmailInput):
    def __init__(self, attrs: _OptAttrs | None = ...) -> None: ...

class AdminURLFieldWidget(forms.URLInput):
    template_name: str
    def __init__(self, attrs: _OptAttrs | None = ..., validator_class: Any = ...) -> None: ...
    def get_context(self, name: str, value: Any, attrs: _OptAttrs | None) -> dict[str, Any]: ...

class AdminIntegerFieldWidget(forms.NumberInput):
    def __init__(self, attrs: _OptAttrs | None = ...) -> None: ...
    class_name: str

class AdminBigIntegerFieldWidget(AdminIntegerFieldWidget):
    class_name: str

class AdminUUIDInputWidget(forms.TextInput):
    def __init__(self, attrs: _OptAttrs | None = ...) -> None: ...

SELECT2_TRANSLATIONS: dict[str, str]

class AutocompleteMixin:
    url_name: str
    field: Any
    admin_site: AdminSite
    db: str | None
    choices: Any
    attrs: _OptAttrs
    def __init__(
        self,
        field: Any,
        admin_site: AdminSite,
        attrs: _OptAttrs | None = ...,
        choices: Any = ...,
        using: str | None = ...,
    ) -> None: ...
    def get_url(self) -> str: ...
    def build_attrs(self, base_attrs: _OptAttrs, extra_attrs: _OptAttrs | None = ...) -> dict[str, Any]: ...
    # typo in source: `attr` instead of `attrs`
    def optgroups(
        self, name: str, value: Sequence[str], attr: _OptAttrs | None = ...
    ) -> list[tuple[str | None, list[dict[str, Any]], int | None]]: ...

class AutocompleteSelect(AutocompleteMixin, forms.Select): ...
class AutocompleteSelectMultiple(AutocompleteMixin, forms.SelectMultiple): ...



import smtplib
import threading

from django.core.mail.backends.base import BaseEmailBackend

class EmailBackend(BaseEmailBackend):
    host: str
    port: int
    username: str
    password: str
    use_tls: bool
    use_ssl: bool
    timeout: int | None
    ssl_keyfile: str | None
    ssl_certfile: str | None
    connection: smtplib.SMTP_SSL | smtplib.SMTP | None
    _lock: threading.RLock



import sys
from typing import Any

class Str: ...

class Xxo:
    def demo(self) -> None: ...

def foo(__i: int, __j: int) -> Any: ...
def new() -> Xxo: ...

if sys.version_info >= (3, 10):
    class Error: ...

else:
    class error: ...
    class Null: ...
    def roj(__b: Any) -> None: ...



from collections.abc import Iterable, Sequence
from typing import Any

from django.core.checks import CheckMessage

def check_database_backends(databases: Iterable[str] | None = ..., **kwargs: Any) -> Sequence[CheckMessage]: ...



from .query import Query as Query
from .query import RawQuery as RawQuery
from .subqueries import AggregateQuery as AggregateQuery
from .subqueries import DeleteQuery as DeleteQuery
from .subqueries import InsertQuery as InsertQuery
from .subqueries import UpdateQuery as UpdateQuery



from typing import Any

def env_func(f: Any, argtypes: Any) -> Any: ...
def pnt_func(f: Any) -> Any: ...
def topology_func(f: Any) -> Any: ...

from_json: Any
to_json: Any
to_kml: Any
getx: Any
gety: Any
getz: Any
from_wkb: Any
from_wkt: Any
from_gml: Any
create_geom: Any
clone_geom: Any
get_geom_ref: Any
get_boundary: Any
geom_convex_hull: Any
geom_diff: Any
geom_intersection: Any
geom_sym_diff: Any
geom_union: Any
add_geom: Any
import_wkt: Any
destroy_geom: Any
to_wkb: Any
to_wkt: Any
to_gml: Any
get_wkbsize: Any
assign_srs: Any
get_geom_srs: Any
get_area: Any
get_centroid: Any
get_dims: Any
get_coord_dim: Any
set_coord_dim: Any
is_empty: Any
get_geom_count: Any
get_geom_name: Any
get_geom_type: Any
get_point_count: Any
get_point: Any
geom_close_rings: Any
ogr_contains: Any
ogr_crosses: Any
ogr_disjoint: Any
ogr_equals: Any
ogr_intersects: Any
ogr_overlaps: Any
ogr_touches: Any
ogr_within: Any
geom_transform: Any
geom_transform_to: Any
get_envelope: Any



from collections.abc import Callable, Mapping, Sequence
from typing import Any, Protocol, TypeVar, overload

from django.db.models import Manager, QuerySet
from django.db.models.base import Model
from django.http import HttpRequest
from django.http.response import HttpResponse, HttpResponsePermanentRedirect, HttpResponseRedirect
from typing_extensions import Literal

def render(
    request: HttpRequest | None,
    template_name: str | Sequence[str],
    context: Mapping[str, Any] | None = ...,
    content_type: str | None = ...,
    status: int | None = ...,
    using: str | None = ...,
) -> HttpResponse: ...

class SupportsGetAbsoluteUrl(Protocol):
    def get_absolute_url(self) -> str: ...

@overload
def redirect(
    to: Callable | str | SupportsGetAbsoluteUrl, *args: Any, permanent: Literal[True], **kwargs: Any
) -> HttpResponsePermanentRedirect: ...
@overload
def redirect(
    to: Callable | str | SupportsGetAbsoluteUrl, *args: Any, permanent: Literal[False] = ..., **kwargs: Any
) -> HttpResponseRedirect: ...
@overload
def redirect(
    to: Callable | str | SupportsGetAbsoluteUrl, *args: Any, permanent: bool, **kwargs: Any
) -> HttpResponseRedirect | HttpResponsePermanentRedirect: ...

_T = TypeVar("_T", bound=Model)

def get_object_or_404(klass: type[_T] | Manager[_T] | QuerySet[_T], *args: Any, **kwargs: Any) -> _T: ...
def get_list_or_404(klass: type[_T] | Manager[_T] | QuerySet[_T], *args: Any, **kwargs: Any) -> list[_T]: ...
def resolve_url(to: Callable | Model | str, *args: Any, **kwargs: Any) -> str: ...



from rest_framework.schemas import SchemaGenerator
from rest_framework.views import APIView

class SchemaView(APIView):
    _ignore_model_permissions: bool
    public: bool
    schema_generator: SchemaGenerator | None



from collections.abc import Generator, Iterator, Sequence
from email import _ParamsType, _ParamType
from email.charset import Charset
from email.contentmanager import ContentManager
from email.errors import MessageDefect
from email.policy import Policy
from typing import Any, TypeVar, overload
from typing_extensions import Self, TypeAlias

__all__ = ["Message", "EmailMessage"]

_T = TypeVar("_T")

_PayloadType: TypeAlias = list[Message] | str | bytes | bytearray
_CharsetType: TypeAlias = Charset | str | None
_HeaderType: TypeAlias = Any

class Message:
    policy: Policy  # undocumented
    preamble: str | None
    epilogue: str | None
    defects: list[MessageDefect]
    def is_multipart(self) -> bool: ...
    def set_unixfrom(self, unixfrom: str) -> None: ...
    def get_unixfrom(self) -> str | None: ...
    def attach(self, payload: Message) -> None: ...
    def get_payload(self, i: int | None = None, decode: bool = False) -> Any: ...  # returns _PayloadType | None
    def set_payload(self, payload: _PayloadType, charset: _CharsetType = None) -> None: ...
    def set_charset(self, charset: _CharsetType) -> None: ...
    def get_charset(self) -> _CharsetType: ...
    def __len__(self) -> int: ...
    def __contains__(self, name: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __getitem__(self, name: str) -> _HeaderType: ...
    def __setitem__(self, name: str, val: _HeaderType) -> None: ...
    def __delitem__(self, name: str) -> None: ...
    def keys(self) -> list[str]: ...
    def values(self) -> list[_HeaderType]: ...
    def items(self) -> list[tuple[str, _HeaderType]]: ...
    @overload
    def get(self, name: str, failobj: None = None) -> _HeaderType | None: ...
    @overload
    def get(self, name: str, failobj: _T) -> _HeaderType | _T: ...
    @overload
    def get_all(self, name: str, failobj: None = None) -> list[_HeaderType] | None: ...
    @overload
    def get_all(self, name: str, failobj: _T) -> list[_HeaderType] | _T: ...
    def add_header(self, _name: str, _value: str, **_params: _ParamsType) -> None: ...
    def replace_header(self, _name: str, _value: _HeaderType) -> None: ...
    def get_content_type(self) -> str: ...
    def get_content_maintype(self) -> str: ...
    def get_content_subtype(self) -> str: ...
    def get_default_type(self) -> str: ...
    def set_default_type(self, ctype: str) -> None: ...
    @overload
    def get_params(
        self, failobj: None = None, header: str = "content-type", unquote: bool = True
    ) -> list[tuple[str, str]] | None: ...
    @overload
    def get_params(self, failobj: _T, header: str = "content-type", unquote: bool = True) -> list[tuple[str, str]] | _T: ...
    @overload
    def get_param(
        self, param: str, failobj: None = None, header: str = "content-type", unquote: bool = True
    ) -> _ParamType | None: ...
    @overload
    def get_param(self, param: str, failobj: _T, header: str = "content-type", unquote: bool = True) -> _ParamType | _T: ...
    def del_param(self, param: str, header: str = "content-type", requote: bool = True) -> None: ...
    def set_type(self, type: str, header: str = "Content-Type", requote: bool = True) -> None: ...
    @overload
    def get_filename(self, failobj: None = None) -> str | None: ...
    @overload
    def get_filename(self, failobj: _T) -> str | _T: ...
    @overload
    def get_boundary(self, failobj: None = None) -> str | None: ...
    @overload
    def get_boundary(self, failobj: _T) -> str | _T: ...
    def set_boundary(self, boundary: str) -> None: ...
    @overload
    def get_content_charset(self) -> str | None: ...
    @overload
    def get_content_charset(self, failobj: _T) -> str | _T: ...
    @overload
    def get_charsets(self, failobj: None = None) -> list[str | None]: ...
    @overload
    def get_charsets(self, failobj: _T) -> list[str | _T]: ...
    def walk(self) -> Generator[Self, None, None]: ...
    def get_content_disposition(self) -> str | None: ...
    def as_string(self, unixfrom: bool = False, maxheaderlen: int = 0, policy: Policy | None = None) -> str: ...
    def as_bytes(self, unixfrom: bool = False, policy: Policy | None = None) -> bytes: ...
    def __bytes__(self) -> bytes: ...
    def set_param(
        self,
        param: str,
        value: str,
        header: str = "Content-Type",
        requote: bool = True,
        charset: str | None = None,
        language: str = "",
        replace: bool = False,
    ) -> None: ...
    def __init__(self, policy: Policy = ...) -> None: ...
    # The following two methods are undocumented, but a source code comment states that they are public API
    def set_raw(self, name: str, value: _HeaderType) -> None: ...
    def raw_items(self) -> Iterator[tuple[str, _HeaderType]]: ...

class MIMEPart(Message):
    def __init__(self, policy: Policy | None = None) -> None: ...
    def get_body(self, preferencelist: Sequence[str] = ("related", "html", "plain")) -> Message | None: ...
    def iter_attachments(self) -> Iterator[Message]: ...
    def iter_parts(self) -> Iterator[Message]: ...
    def get_content(self, *args: Any, content_manager: ContentManager | None = None, **kw: Any) -> Any: ...
    def set_content(self, *args: Any, content_manager: ContentManager | None = None, **kw: Any) -> None: ...
    def make_related(self, boundary: str | None = None) -> None: ...
    def make_alternative(self, boundary: str | None = None) -> None: ...
    def make_mixed(self, boundary: str | None = None) -> None: ...
    def add_related(self, *args: Any, content_manager: ContentManager | None = ..., **kw: Any) -> None: ...
    def add_alternative(self, *args: Any, content_manager: ContentManager | None = ..., **kw: Any) -> None: ...
    def add_attachment(self, *args: Any, content_manager: ContentManager | None = ..., **kw: Any) -> None: ...
    def clear(self) -> None: ...
    def clear_content(self) -> None: ...
    def as_string(self, unixfrom: bool = False, maxheaderlen: int | None = None, policy: Policy | None = None) -> str: ...
    def is_attachment(self) -> bool: ...

class EmailMessage(MIMEPart): ...



from os import path
from typing import List

from django.core.management import call_command

from nwon_django_toolbox.typings.test_fixture import Fixture


def load_fixtures(fixture_files: List[Fixture], app_label: str):
    for fixture in fixture_files:
        if path.isfile(fixture.path):
            call_command(
                "loaddata",
                fixture.path,
                app_label=app_label,
            )


__all__ = [
    "load_fixtures",
]



import datetime
from typing import List, Union

import pytz
from django.core.exceptions import ValidationError
from django.db.models import Model

from nwon_django_toolbox.typings import AllowedUpdate


def validate_model_update(
    old_model_instance: Model,
    updated_model_instance: Model,
    allowed_updates: Union[List[AllowedUpdate], None],
):
    """
    Checks an update model instance against an allowed set of fields and their values
    """

    if updated_model_instance._state.adding is True:
        return

    if not allowed_updates:
        return

    fields = updated_model_instance._meta.get_fields()

    allowed_field_names = [update.field_name for update in allowed_updates]

    if old_model_instance.__class__ != updated_model_instance.__class__:
        raise ValidationError(
            f"You tried to validate update for two instances of a different class: "
            f"{old_model_instance.__class__}  and "
            f"{updated_model_instance.__class__}"
        )

    if old_model_instance.pk != updated_model_instance.pk:
        raise ValidationError(
            f"You tried to validate update for two instances of "
            f"{old_model_instance.__class__} with a different pk value: "
            f"{old_model_instance.pk} and {updated_model_instance.pk}"
        )

    for field in fields:
        field_name = field.name
        old_value = getattr(old_model_instance, field_name)
        new_value = getattr(updated_model_instance, field_name)

        if isinstance(old_value, datetime.datetime) and old_value.tzinfo is None:
            old_value = pytz.utc.localize(old_value)

        if isinstance(new_value, datetime.datetime) and new_value.tzinfo is None:
            new_value = pytz.utc.localize(new_value)

        if old_value == new_value:
            continue

        if not field_name in allowed_field_names:
            raise ValidationError(
                {
                    field_name: f"You are not allowed to update {field_name} "
                    f"with value {new_value} from {old_value} for "
                    f"{updated_model_instance.__class__} "
                    f"with pk {updated_model_instance.pk}"
                }
            )

        allowed_updates_for_field_name = _filter_allowed_updates(
            allowed_updates, field_name
        )

        # There is some configuration which limits possible field values
        new_value_has_constraint = any(
            x.allowed_values is not None for x in allowed_updates_for_field_name
        )

        # The new value is explicitly allowed in some field configuration
        new_value_matches_constraint = any(
            new_value in x.allowed_values
            for x in allowed_updates_for_field_name
            if x.allowed_values
        )

        if new_value and new_value_has_constraint and not new_value_matches_constraint:
            raise ValidationError(
                {
                    field_name: f"{new_value} can't be set as value for "
                    f"{field_name} in {updated_model_instance} with pk "
                    f"{updated_model_instance.pk}"
                }
            )


def _filter_allowed_updates(
    allowed_updates: List[AllowedUpdate], field_name: str
) -> List[AllowedUpdate]:
    return list(
        filter(
            lambda x: field_name == x.field_name and x.allowed_values,
            allowed_updates,
        )
    )



from typing import Any

from django.contrib.gis.db import models
from django.contrib.gis.db.backends.base.models import SpatialRefSysMixin

class OracleGeometryColumns(models.Model):
    table_name: Any
    column_name: Any
    srid: Any

    class Meta:
        app_label: str
        db_table: str
        managed: bool
    @classmethod
    def table_name_col(cls) -> Any: ...
    @classmethod
    def geom_col_name(cls) -> Any: ...

class OracleSpatialRefSys(models.Model, SpatialRefSysMixin):
    cs_name: Any
    srid: Any
    auth_srid: Any
    auth_name: Any
    wktext: Any
    cs_bounds: Any

    class Meta:
        app_label: str
        db_table: str
        managed: bool
    @property
    def wkt(self) -> Any: ...



from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations import AddConstraint, AddIndex, RemoveIndex
from django.db.migrations.operations.base import Operation
from typing_extensions import Literal

class CreateExtension(Operation):
    reversible: bool
    name: str
    def __init__(self, name: str) -> None: ...
    def extension_exists(self, schema_editor: BaseDatabaseSchemaEditor, extension: str) -> bool: ...

class BloomExtension(CreateExtension):
    def __init__(self) -> None: ...

class BtreeGinExtension(CreateExtension):
    def __init__(self) -> None: ...

class BtreeGistExtension(CreateExtension):
    def __init__(self) -> None: ...

class CITextExtension(CreateExtension):
    def __init__(self) -> None: ...

class CryptoExtension(CreateExtension):
    def __init__(self) -> None: ...

class HStoreExtension(CreateExtension):
    def __init__(self) -> None: ...

class TrigramExtension(CreateExtension):
    def __init__(self) -> None: ...

class UnaccentExtension(CreateExtension):
    def __init__(self) -> None: ...

class NotInTransactionMixin:
    def _ensure_not_in_transaction(self, schema_editor: BaseDatabaseSchemaEditor) -> None: ...

class AddIndexConcurrently(NotInTransactionMixin, AddIndex):
    atomic: Literal[False]

class RemoveIndexConcurrently(NotInTransactionMixin, RemoveIndex):
    atomic: Literal[False]

class CollationOperation(Operation):
    name: str
    locale: str
    provider: str
    deterministic: bool
    def __init__(self, name: str, locale: str, *, provider: str = ..., deterministic: bool = ...) -> None: ...
    def create_collation(self, schema_editor: BaseDatabaseSchemaEditor) -> None: ...
    def remove_collation(self, schema_editor: BaseDatabaseSchemaEditor) -> None: ...

class CreateCollation(CollationOperation): ...
class RemoveCollation(CollationOperation): ...
class AddConstraintNotValid(AddConstraint): ...

class ValidateConstraint(Operation):
    model_name: str
    name: str
    def __init__(self, model_name: str, name: str) -> None: ...



from typing import Any

from django import template
from django.template.base import Parser, Token
from django.template.context import Context

register: Any

class FlatpageNode(template.Node):
    context_name: str
    starts_with: None
    user: None
    def __init__(self, context_name: str, starts_with: str | None = ..., user: str | None = ...) -> None: ...
    def render(self, context: Context) -> str: ...

def get_flatpages(parser: Parser, token: Token) -> FlatpageNode: ...



from typing import Any, ClassVar
from typing_extensions import Literal

from ._imaging import _PixelAccessor
from .ImageFile import ImageFile

COMPRESSION: Any
PAD: Any

def i(c): ...
def dump(c) -> None: ...

class IptcImageFile(ImageFile):
    format: ClassVar[Literal["IPTC"]]
    format_description: ClassVar[str]
    def getint(self, key): ...
    def field(self): ...
    im: Any
    def load(self) -> _PixelAccessor: ...

def getiptcinfo(im): ...



from nwon_django_toolbox.signals.disable_signals import DisableSignals
from nwon_django_toolbox.signals.signals import (
    DISABLE_SIGNALS,
    disable_signals,
    enable_signals,
)

__ALL__ = [
    "DisableSignals",
    "DISABLE_SIGNALS",
    "disable_signals",
    "enable_signals",
]



from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.http.request import HttpRequest
from django.utils.deprecation import MiddlewareMixin

def get_user(request: HttpRequest) -> AnonymousUser | AbstractBaseUser: ...

class AuthenticationMiddleware(MiddlewareMixin):
    def process_request(self, request: HttpRequest) -> None: ...

class RemoteUserMiddleware(MiddlewareMixin):
    header: str
    force_logout_if_no_header: bool
    def process_request(self, request: HttpRequest) -> None: ...
    def clean_username(self, username: str, request: HttpRequest) -> str: ...

class PersistentRemoteUserMiddleware(RemoteUserMiddleware):
    force_logout_if_no_header: bool



from collections.abc import Sequence
from typing import Any

from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.backends.ddl_references import Statement
from django.db.models import Func, Index, Model
from django.db.models.expressions import BaseExpression, Combinable
from django.db.models.query_utils import Q
from django.utils.datastructures import _ListOrTuple

class PostgresIndex(Index):
    @property
    def max_name_length(self) -> int: ...  # type: ignore
    def create_sql(
        self, model: type[Model], schema_editor: BaseDatabaseSchemaEditor, using: str = ..., **kwargs: Any
    ) -> Statement: ...
    def check_supported(self, schema_editor: BaseDatabaseSchemaEditor) -> None: ...
    def get_with_params(self) -> Sequence[str]: ...

class BloomIndex(PostgresIndex):
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        length: int | None = ...,
        columns: _ListOrTuple[int] = ...,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...

class BrinIndex(PostgresIndex):
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        autosummarize: bool | None = ...,
        pages_per_range: int | None = ...,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...

class BTreeIndex(PostgresIndex):
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        fillfactor: int | None = ...,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...

class GinIndex(PostgresIndex):
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        fastupdate: bool | None = ...,
        gin_pending_list_limit: int | None = ...,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...

class GistIndex(PostgresIndex):
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        buffering: bool | None = ...,
        fillfactor: int | None = ...,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...

class HashIndex(PostgresIndex):
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        fillfactor: int | None = ...,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...

class SpGistIndex(PostgresIndex):
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        fillfactor: int | None = ...,
        fields: Sequence[str] = ...,
        name: str | None = ...,
        db_tablespace: str | None = ...,
        opclasses: Sequence[str] = ...,
        condition: Q | None = ...,
        include: Sequence[str] | None = ...,
    ) -> None: ...

class OpClass(Func):
    def __init__(
        self,
        expression: BaseExpression | Combinable | str,
        name: str,
    ) -> None: ...



from collections.abc import Iterator, Mapping
from re import Pattern
from typing import TypeVar

ESCAPE_MAPPINGS: Mapping[str, str | None]

class Choice(list): ...
class Group(list): ...
class NonCapture(list): ...

def normalize(pattern: str) -> list[tuple[str, list[str]]]: ...
def next_char(input_iter: Iterator[str]) -> Iterator[tuple[str, bool]]: ...
def walk_to_end(ch: str, input_iter: Iterator[tuple[str, bool]]) -> None: ...
def get_quantifier(ch: str, input_iter: Iterator[tuple[str, bool]]) -> tuple[int, str | None]: ...
def contains(source: Group | NonCapture | str, inst: type[Group]) -> bool: ...
def flatten_result(
    source: list[Choice | Group | str] | Group | NonCapture | None,
) -> tuple[list[str], list[list[str]]]: ...

# Returns SimpleLazyObject, but we can safely ignore it
_P = TypeVar("_P", str, bytes)

def _lazy_re_compile(regex: _P | Pattern[_P], flags: int = ...) -> Pattern[_P]: ...



import ssl
import sys
from _typeshed import FileDescriptorLike, ReadableBuffer, WriteableBuffer
from asyncio.events import AbstractEventLoop, AbstractServer, Handle, TimerHandle, _TaskFactory
from asyncio.futures import Future
from asyncio.protocols import BaseProtocol
from asyncio.tasks import Task
from asyncio.transports import BaseTransport, DatagramTransport, ReadTransport, SubprocessTransport, Transport, WriteTransport
from collections.abc import Awaitable, Callable, Coroutine, Generator, Iterable, Sequence
from contextvars import Context
from socket import AddressFamily, SocketKind, _Address, _RetAddress, socket
from typing import IO, Any, TypeVar, overload
from typing_extensions import Literal, TypeAlias

if sys.version_info >= (3, 9):
    __all__ = ("BaseEventLoop", "Server")
else:
    __all__ = ("BaseEventLoop",)

_T = TypeVar("_T")
_ProtocolT = TypeVar("_ProtocolT", bound=BaseProtocol)
_Context: TypeAlias = dict[str, Any]
_ExceptionHandler: TypeAlias = Callable[[AbstractEventLoop, _Context], object]
_ProtocolFactory: TypeAlias = Callable[[], BaseProtocol]
_SSLContext: TypeAlias = bool | None | ssl.SSLContext

class Server(AbstractServer):
    if sys.version_info >= (3, 11):
        def __init__(
            self,
            loop: AbstractEventLoop,
            sockets: Iterable[socket],
            protocol_factory: _ProtocolFactory,
            ssl_context: _SSLContext,
            backlog: int,
            ssl_handshake_timeout: float | None,
            ssl_shutdown_timeout: float | None = None,
        ) -> None: ...
    else:
        def __init__(
            self,
            loop: AbstractEventLoop,
            sockets: Iterable[socket],
            protocol_factory: _ProtocolFactory,
            ssl_context: _SSLContext,
            backlog: int,
            ssl_handshake_timeout: float | None,
        ) -> None: ...

    def get_loop(self) -> AbstractEventLoop: ...
    def is_serving(self) -> bool: ...
    async def start_serving(self) -> None: ...
    async def serve_forever(self) -> None: ...
    if sys.version_info >= (3, 8):
        @property
        def sockets(self) -> tuple[socket, ...]: ...
    else:
        @property
        def sockets(self) -> list[socket]: ...

    def close(self) -> None: ...
    async def wait_closed(self) -> None: ...

class BaseEventLoop(AbstractEventLoop):
    def run_forever(self) -> None: ...
    # Can't use a union, see mypy issue  # 1873.
    @overload
    def run_until_complete(self, future: Generator[Any, None, _T]) -> _T: ...
    @overload
    def run_until_complete(self, future: Awaitable[_T]) -> _T: ...
    def stop(self) -> None: ...
    def is_running(self) -> bool: ...
    def is_closed(self) -> bool: ...
    def close(self) -> None: ...
    async def shutdown_asyncgens(self) -> None: ...
    # Methods scheduling callbacks.  All these return Handles.
    def call_soon(self, callback: Callable[..., object], *args: Any, context: Context | None = None) -> Handle: ...
    def call_later(
        self, delay: float, callback: Callable[..., object], *args: Any, context: Context | None = None
    ) -> TimerHandle: ...
    def call_at(
        self, when: float, callback: Callable[..., object], *args: Any, context: Context | None = None
    ) -> TimerHandle: ...
    def time(self) -> float: ...
    # Future methods
    def create_future(self) -> Future[Any]: ...
    # Tasks methods
    if sys.version_info >= (3, 11):
        def create_task(
            self, coro: Coroutine[Any, Any, _T] | Generator[Any, None, _T], *, name: object = None, context: Context | None = None
        ) -> Task[_T]: ...
    elif sys.version_info >= (3, 8):
        def create_task(self, coro: Coroutine[Any, Any, _T] | Generator[Any, None, _T], *, name: object = None) -> Task[_T]: ...
    else:
        def create_task(self, coro: Coroutine[Any, Any, _T] | Generator[Any, None, _T]) -> Task[_T]: ...

    def set_task_factory(self, factory: _TaskFactory | None) -> None: ...
    def get_task_factory(self) -> _TaskFactory | None: ...
    # Methods for interacting with threads
    def call_soon_threadsafe(self, callback: Callable[..., object], *args: Any, context: Context | None = None) -> Handle: ...
    def run_in_executor(self, executor: Any, func: Callable[..., _T], *args: Any) -> Future[_T]: ...
    def set_default_executor(self, executor: Any) -> None: ...
    # Network I/O methods returning Futures.
    async def getaddrinfo(
        self,
        host: bytes | str | None,
        port: bytes | str | int | None,
        *,
        family: int = 0,
        type: int = 0,
        proto: int = 0,
        flags: int = 0,
    ) -> list[tuple[AddressFamily, SocketKind, int, str, tuple[str, int] | tuple[str, int, int, int]]]: ...
    async def getnameinfo(self, sockaddr: tuple[str, int] | tuple[str, int, int, int], flags: int = 0) -> tuple[str, str]: ...
    if sys.version_info >= (3, 11):
        @overload
        async def create_connection(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            host: str = ...,
            port: int = ...,
            *,
            ssl: _SSLContext = None,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            sock: None = None,
            local_addr: tuple[str, int] | None = None,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
            happy_eyeballs_delay: float | None = None,
            interleave: int | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...
        @overload
        async def create_connection(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            host: None = None,
            port: None = None,
            *,
            ssl: _SSLContext = None,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            sock: socket,
            local_addr: None = None,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
            happy_eyeballs_delay: float | None = None,
            interleave: int | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...
    elif sys.version_info >= (3, 8):
        @overload
        async def create_connection(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            host: str = ...,
            port: int = ...,
            *,
            ssl: _SSLContext = None,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            sock: None = None,
            local_addr: tuple[str, int] | None = None,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
            happy_eyeballs_delay: float | None = None,
            interleave: int | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...
        @overload
        async def create_connection(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            host: None = None,
            port: None = None,
            *,
            ssl: _SSLContext = None,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            sock: socket,
            local_addr: None = None,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
            happy_eyeballs_delay: float | None = None,
            interleave: int | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...
    else:
        @overload
        async def create_connection(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            host: str = ...,
            port: int = ...,
            *,
            ssl: _SSLContext = None,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            sock: None = None,
            local_addr: tuple[str, int] | None = None,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...
        @overload
        async def create_connection(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            host: None = None,
            port: None = None,
            *,
            ssl: _SSLContext = None,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            sock: socket,
            local_addr: None = None,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...
    if sys.version_info >= (3, 11):
        @overload
        async def create_server(
            self,
            protocol_factory: _ProtocolFactory,
            host: str | Sequence[str] | None = None,
            port: int = ...,
            *,
            family: int = ...,
            flags: int = ...,
            sock: None = None,
            backlog: int = 100,
            ssl: _SSLContext = None,
            reuse_address: bool | None = None,
            reuse_port: bool | None = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
            start_serving: bool = True,
        ) -> Server: ...
        @overload
        async def create_server(
            self,
            protocol_factory: _ProtocolFactory,
            host: None = None,
            port: None = None,
            *,
            family: int = ...,
            flags: int = ...,
            sock: socket = ...,
            backlog: int = 100,
            ssl: _SSLContext = None,
            reuse_address: bool | None = None,
            reuse_port: bool | None = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
            start_serving: bool = True,
        ) -> Server: ...
        async def start_tls(
            self,
            transport: BaseTransport,
            protocol: BaseProtocol,
            sslcontext: ssl.SSLContext,
            *,
            server_side: bool = False,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
        ) -> Transport: ...
        async def connect_accepted_socket(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            sock: socket,
            *,
            ssl: _SSLContext = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...
    else:
        @overload
        async def create_server(
            self,
            protocol_factory: _ProtocolFactory,
            host: str | Sequence[str] | None = None,
            port: int = ...,
            *,
            family: int = ...,
            flags: int = ...,
            sock: None = None,
            backlog: int = 100,
            ssl: _SSLContext = None,
            reuse_address: bool | None = None,
            reuse_port: bool | None = None,
            ssl_handshake_timeout: float | None = None,
            start_serving: bool = True,
        ) -> Server: ...
        @overload
        async def create_server(
            self,
            protocol_factory: _ProtocolFactory,
            host: None = None,
            port: None = None,
            *,
            family: int = ...,
            flags: int = ...,
            sock: socket = ...,
            backlog: int = 100,
            ssl: _SSLContext = None,
            reuse_address: bool | None = None,
            reuse_port: bool | None = None,
            ssl_handshake_timeout: float | None = None,
            start_serving: bool = True,
        ) -> Server: ...
        async def start_tls(
            self,
            transport: BaseTransport,
            protocol: BaseProtocol,
            sslcontext: ssl.SSLContext,
            *,
            server_side: bool = False,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
        ) -> Transport: ...
        async def connect_accepted_socket(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            sock: socket,
            *,
            ssl: _SSLContext = None,
            ssl_handshake_timeout: float | None = None,
        ) -> tuple[Transport, _ProtocolT]: ...

    async def sock_sendfile(
        self, sock: socket, file: IO[bytes], offset: int = 0, count: int | None = None, *, fallback: bool | None = True
    ) -> int: ...
    async def sendfile(
        self, transport: WriteTransport, file: IO[bytes], offset: int = 0, count: int | None = None, *, fallback: bool = True
    ) -> int: ...
    if sys.version_info >= (3, 11):
        async def create_datagram_endpoint(  # type: ignore[override]
            self,
            protocol_factory: Callable[[], _ProtocolT],
            local_addr: tuple[str, int] | str | None = None,
            remote_addr: tuple[str, int] | str | None = None,
            *,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            reuse_port: bool | None = None,
            allow_broadcast: bool | None = None,
            sock: socket | None = None,
        ) -> tuple[DatagramTransport, _ProtocolT]: ...
    else:
        async def create_datagram_endpoint(
            self,
            protocol_factory: Callable[[], _ProtocolT],
            local_addr: tuple[str, int] | str | None = None,
            remote_addr: tuple[str, int] | str | None = None,
            *,
            family: int = 0,
            proto: int = 0,
            flags: int = 0,
            reuse_address: bool | None = ...,
            reuse_port: bool | None = None,
            allow_broadcast: bool | None = None,
            sock: socket | None = None,
        ) -> tuple[DatagramTransport, _ProtocolT]: ...
    # Pipes and subprocesses.
    async def connect_read_pipe(
        self, protocol_factory: Callable[[], _ProtocolT], pipe: Any
    ) -> tuple[ReadTransport, _ProtocolT]: ...
    async def connect_write_pipe(
        self, protocol_factory: Callable[[], _ProtocolT], pipe: Any
    ) -> tuple[WriteTransport, _ProtocolT]: ...
    async def subprocess_shell(
        self,
        protocol_factory: Callable[[], _ProtocolT],
        cmd: bytes | str,
        *,
        stdin: int | IO[Any] | None = -1,
        stdout: int | IO[Any] | None = -1,
        stderr: int | IO[Any] | None = -1,
        universal_newlines: Literal[False] = False,
        shell: Literal[True] = True,
        bufsize: Literal[0] = 0,
        encoding: None = None,
        errors: None = None,
        text: Literal[False, None] = None,
        **kwargs: Any,
    ) -> tuple[SubprocessTransport, _ProtocolT]: ...
    async def subprocess_exec(
        self,
        protocol_factory: Callable[[], _ProtocolT],
        program: Any,
        *args: Any,
        stdin: int | IO[Any] | None = -1,
        stdout: int | IO[Any] | None = -1,
        stderr: int | IO[Any] | None = -1,
        universal_newlines: Literal[False] = False,
        shell: Literal[False] = False,
        bufsize: Literal[0] = 0,
        encoding: None = None,
        errors: None = None,
        **kwargs: Any,
    ) -> tuple[SubprocessTransport, _ProtocolT]: ...
    def add_reader(self, fd: FileDescriptorLike, callback: Callable[..., Any], *args: Any) -> None: ...
    def remove_reader(self, fd: FileDescriptorLike) -> bool: ...
    def add_writer(self, fd: FileDescriptorLike, callback: Callable[..., Any], *args: Any) -> None: ...
    def remove_writer(self, fd: FileDescriptorLike) -> bool: ...
    # The sock_* methods (and probably some others) are not actually implemented on
    # BaseEventLoop, only on subclasses. We list them here for now for convenience.
    async def sock_recv(self, sock: socket, nbytes: int) -> bytes: ...
    async def sock_recv_into(self, sock: socket, buf: WriteableBuffer) -> int: ...
    async def sock_sendall(self, sock: socket, data: ReadableBuffer) -> None: ...
    async def sock_connect(self, sock: socket, address: _Address) -> None: ...
    async def sock_accept(self, sock: socket) -> tuple[socket, _RetAddress]: ...
    if sys.version_info >= (3, 11):
        async def sock_recvfrom(self, sock: socket, bufsize: int) -> tuple[bytes, _RetAddress]: ...
        async def sock_recvfrom_into(self, sock: socket, buf: WriteableBuffer, nbytes: int = 0) -> tuple[int, _RetAddress]: ...
        async def sock_sendto(self, sock: socket, data: ReadableBuffer, address: _Address) -> int: ...
    # Signal handling.
    def add_signal_handler(self, sig: int, callback: Callable[..., Any], *args: Any) -> None: ...
    def remove_signal_handler(self, sig: int) -> bool: ...
    # Error handlers.
    def set_exception_handler(self, handler: _ExceptionHandler | None) -> None: ...
    def get_exception_handler(self) -> _ExceptionHandler | None: ...
    def default_exception_handler(self, context: _Context) -> None: ...
    def call_exception_handler(self, context: _Context) -> None: ...
    # Debug flag management.
    def get_debug(self) -> bool: ...
    def set_debug(self, enabled: bool) -> None: ...
    if sys.version_info >= (3, 9):
        async def shutdown_default_executor(self) -> None: ...



import copy
from typing import Any, List, Optional, Type

from django.db.models.base import Model
from rest_framework.serializers import Serializer

from nwon_django_toolbox.tests.api_client.api_test import ApiTest
from nwon_django_toolbox.tests.helper.check_object_against_parameter import (
    check_object_against_parameter,
)
from nwon_django_toolbox.tests.helper.dictionary_is_serialized_instance import (
    dictionary_is_serialized_instance,
)
from nwon_django_toolbox.typings import RequestBodyFormat
from nwon_django_toolbox.url_helper import (
    detail_url_for_model,
    detail_url_for_model_class,
    list_url_for_model_class,
)


def check_post_basics(
    model_class: Type[Model],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
    needs_authentication: bool = True,
):
    """
    Helper function that checks basic functionality of a post endpoint

    1. Method not allowed on detail url if needs_authentication == True
    2. Unauthenticated on detail url if needs_authentication == True
    3. Method not allowed on detail url if needs_authentication == True and token is set
    """

    api_test = ApiTest()
    url = detail_url_for_model_class(model_class, "some-non-existing-id")

    if needs_authentication:
        api_test.post_unauthorized(url, {}, body_format)

        if authentication_token:
            api_test.set_bearer_token(authentication_token)
            api_test.post_method_not_allowed(url, {}, body_format)
    else:
        api_test.post_method_not_allowed(url, {}, body_format)


def check_post_not_allowed(
    model_class: Type[Model],
    authentication_token: Optional[str] = None,
):
    """
    Test that post is not allowed
    """

    list_url = list_url_for_model_class(model_class)

    api_test = ApiTest(token=authentication_token)
    api_test.post_method_not_allowed(list_url, {})


def check_post_parameters_successful(
    model_class: Type[Model],
    successful_parameters: List[dict],
    authentication_token: Optional[str] = None,
    return_serializer: Optional[Type[Serializer]] = None,
    body_format=RequestBodyFormat.Json,
):
    """
    Test that a list of parameter create successfully.

    1. Successful create
    2. Checks the request parameters against the response.
    3. Checks the request parameters against the object returned from get endpoint.
    (4.) Optionally tests the response against a serializer
    """

    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    for successful_parameter in successful_parameters:
        response = api_test.post_create_successful(
            list_url, successful_parameter, body_format=body_format
        )
        check_object_against_parameter(response, successful_parameter)

        created_object = model_class.objects.last()
        assert isinstance(created_object, model_class)

        if return_serializer:
            assert dictionary_is_serialized_instance(
                instance=created_object,
                serializer=return_serializer,
                response=response,
            )

        url = detail_url_for_model(created_object)
        object_to_check = api_test.get_successful(url)
        check_object_against_parameter(object_to_check, successful_parameter)


def check_post_parameters_required(
    model_class: Type[Model],
    successful_parameter: dict,
    required_keys: List[str],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    """
    Test whether a certain parameter is required. First a successful requests verifies
    the parameters. Afterwards the key that should be required is left out and should
    lead to a failing request.
    """

    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    # make sure that parameter are working
    api_test.post_create_successful(
        list_url, successful_parameter, body_format=body_format
    )

    for key in required_keys:
        parameter = copy.deepcopy(successful_parameter)
        parameter.pop(key)
        api_test.post_bad_request(list_url, parameter, body_format=body_format)


def check_post_parameters_not_required(
    model_class: Type[Model],
    successful_parameter: dict,
    required_keys: List[str],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    """
    Test whether a certain parameter is NOT required. First a successful requests verifies
    the parameters. Afterwards the key that shouldn't be required is left out and should
    still lead to a successful request.
    """

    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    # make sure that parameter are working
    api_test.post_create_successful(
        list_url, successful_parameter, body_format=body_format
    )

    for key in required_keys:
        parameter = copy.deepcopy(successful_parameter)
        parameter.pop(key)
        api_test.post_create_successful(list_url, parameter, body_format=body_format)


def check_post_parameters_failing(
    model_class: Type[Model],
    failing_parameters: List[dict],
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    """
    Test whether certain parameters are failing.
    """

    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    for failing_parameter in failing_parameters:
        api_test.post_bad_request(list_url, failing_parameter, body_format=body_format)


def check_post_read_only_field(
    model_class: Type[Model],
    successful_post_parameter: dict,
    key: str,
    value: Any,
    authentication_token: Optional[str] = None,
    body_format=RequestBodyFormat.Json,
):
    """
    Test whether the inclusion of some parameter does not have an effect
    """

    successful_post_parameter[key] = value

    list_url = list_url_for_model_class(model_class)
    api_test = ApiTest(token=authentication_token)

    created_object = api_test.post_create_successful(
        list_url, successful_post_parameter, body_format=body_format
    )

    # check whether read only key was set
    assert created_object[key] != value


__all__ = [
    "check_post_basics",
    "check_post_not_allowed",
    "check_post_parameters_failing",
    "check_post_parameters_successful",
    "check_post_read_only_field",
    "check_post_parameters_not_required",
]



from collections.abc import Callable
from typing import Any, TypeVar, overload

_T = TypeVar("_T")
_TCallable = TypeVar("_TCallable", bound=Callable[..., Any])

@overload
def deconstructible(_type: type[_T]) -> type[_T]: ...
@overload
def deconstructible(*, path: str | None = ...) -> Callable[[_TCallable], _TCallable]: ...



from collections.abc import Mapping, Sequence
from re import Pattern
from typing import Any

from rest_framework.request import Request

class BaseVersioning:
    default_version: str | None
    allowed_versions: Sequence[str] | None
    version_param: str
    def determine_version(self, request: Request, *args: Any, **kwargs: Any) -> str: ...
    def reverse(
        self,
        viewname: str,
        args: Sequence[Any] | None = ...,
        kwargs: Mapping[str, Any] | None = ...,
        request: Request | None = ...,
        format: str | None = ...,
        **extra: Any
    ) -> str: ...
    def is_allowed_version(self, version: str | None) -> bool: ...

class AcceptHeaderVersioning(BaseVersioning):
    invalid_version_message: str

class URLPathVersioning(BaseVersioning):
    invalid_version_message: str

class NamespaceVersioning(BaseVersioning):
    invalid_version_message: str
    def get_versioned_viewname(self, viewname: str, request: Request) -> str: ...

class HostNameVersioning(BaseVersioning):
    hostname_regex: Pattern
    invalid_version_message: str

class QueryParameterVersioning(BaseVersioning):
    invalid_version_message: str



from collections.abc import Iterable, Sequence
from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper

class BaseDatabaseClient:
    executable_name: str | None
    connection: BaseDatabaseWrapper
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    @classmethod
    def settings_to_cmd_args_env(
        cls,
        settings_dict: dict[str, Any],
        parameters: Iterable[str],
    ) -> tuple[Sequence[str], dict[str, str] | None]: ...
    def runshell(self, parameters: Iterable[str]) -> None: ...



from contextlib import ContextDecorator
from datetime import date, datetime, time, timedelta, timezone, tzinfo
from types import TracebackType
from typing import Any, overload

import pytz
from pytz import BaseTzInfo
from typing_extensions import Literal, TypeAlias, TypeGuard

_PytzTzInfoT: TypeAlias = pytz.tzinfo.BaseTzInfo | pytz._FixedOffset

_TzInfoT: TypeAlias = _PytzTzInfoT | tzinfo

utc: Any

def get_fixed_timezone(offset: timedelta | int) -> timezone: ...
def get_default_timezone() -> BaseTzInfo: ...
def get_default_timezone_name() -> str: ...

# Strictly speaking, it is possible to activate() a non-pytz timezone,
# in which case BaseTzInfo is incorrect. However, this is unlikely,
# so we use it anyway, to keep things ergonomic for most users.
def get_current_timezone() -> BaseTzInfo: ...
def get_current_timezone_name() -> str: ...
def activate(timezone: _TzInfoT | str) -> None: ...
def deactivate() -> None: ...

class override(ContextDecorator):
    timezone: str | _TzInfoT | None
    old_timezone: _TzInfoT | None
    def __init__(self, timezone: str | _TzInfoT | None) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None: ...

def localtime(value: datetime | None = ..., timezone: _TzInfoT | None = ...) -> datetime: ...
def localdate(value: datetime | None = ..., timezone: _TzInfoT | None = ...) -> date: ...
def now() -> datetime: ...
@overload
def is_aware(value: time) -> Literal[False]: ...
@overload
def is_aware(value: datetime) -> bool: ...
@overload
def is_naive(value: time) -> Literal[True]: ...
@overload
def is_naive(value: datetime) -> bool: ...
def make_aware(value: datetime, timezone: _TzInfoT | None = ..., is_dst: bool | None = ...) -> datetime: ...
def make_naive(value: datetime, timezone: _TzInfoT | None = ...) -> datetime: ...
def _is_pytz_zone(tz: _TzInfoT) -> TypeGuard[_PytzTzInfoT]: ...



import sys
from _typeshed import StrOrBytesPath
from collections.abc import Callable, Sequence
from typing_extensions import SupportsIndex

if sys.platform != "win32":
    def cloexec_pipe() -> tuple[int, int]: ...
    def fork_exec(
        __process_args: Sequence[StrOrBytesPath] | None,
        __executable_list: Sequence[bytes],
        __close_fds: bool,
        __fds_to_keep: tuple[int, ...],
        __cwd_obj: str,
        __env_list: Sequence[bytes] | None,
        __p2cread: int,
        __p2cwrite: int,
        __c2pred: int,
        __c2pwrite: int,
        __errread: int,
        __errwrite: int,
        __errpipe_read: int,
        __errpipe_write: int,
        __restore_signals: int,
        __call_setsid: int,
        __pgid_to_set: int,
        __gid_object: SupportsIndex | None,
        __groups_list: list[int] | None,
        __uid_object: SupportsIndex | None,
        __child_umask: int,
        __preexec_fn: Callable[[], None],
        __allow_vfork: bool,
    ) -> int: ...



from typing import List, NamedTuple

class struct_spwd(NamedTuple):
    sp_namp: str
    sp_pwdp: str
    sp_lstchg: int
    sp_min: int
    sp_max: int
    sp_warn: int
    sp_inact: int
    sp_expire: int
    sp_flag: int

def getspall() -> List[struct_spwd]: ...
def getspnam(__arg: str) -> struct_spwd: ...



from ctypes import c_char_p
from typing import Any

from django.contrib.gis.geos.libgeos import GEOSFuncFactory

c_uchar_p: Any

class geos_char_p(c_char_p): ...

class GeomOutput(GEOSFuncFactory):
    restype: Any
    errcheck: Any

class IntFromGeom(GEOSFuncFactory):
    argtypes: Any
    restype: Any
    errcheck: Any

class StringFromGeom(GEOSFuncFactory):
    argtypes: Any
    restype: Any
    errcheck: Any

geos_normalize: Any
geos_type: Any
geos_typeid: Any
get_dims: Any
get_num_coords: Any
get_num_geoms: Any
create_point: Any
create_linestring: Any
create_linearring: Any
create_polygon: Any
create_empty_polygon: Any
create_collection: Any
get_extring: Any
get_intring: Any
get_nrings: Any
get_geomn: Any
geom_clone: Any
destroy_geom: Any
geos_get_srid: Any
geos_set_srid: Any



from pathlib import Path
from typing import Any

from django.utils.autoreload import BaseReloader

def watch_for_translation_changes(sender: BaseReloader, **kwargs: Any) -> None: ...
def translation_file_changed(sender: BaseReloader | None, file_path: Path, **kwargs: Any) -> bool: ...



from typing import Any

from _typeshed import Self
from django.contrib.gis.gdal import CoordTransform, SpatialReference
from django.contrib.gis.gdal.geometries import OGRGeometry
from django.contrib.gis.geos.base import GEOSBase
from django.contrib.gis.geos.coordseq import GEOSCoordSeq
from django.contrib.gis.geos.mutable_list import ListMixin
from django.contrib.gis.geos.point import Point
from django.contrib.gis.geos.prepared import PreparedGeometry

class GEOSGeometryBase(GEOSBase):
    ptr_type: Any
    destructor: Any
    has_cs: bool
    def __init__(self, ptr: Any, cls: Any) -> None: ...
    def __copy__(self: Self) -> Self: ...
    def __deepcopy__(self: Self, memodict: Any) -> Self: ...
    @staticmethod
    def from_ewkt(ewkt: str) -> GEOSGeometry: ...
    @classmethod
    def from_gml(cls, gml_string: str) -> GEOSGeometry: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __or__(self, other: GEOSGeometry) -> GEOSGeometry: ...
    def __and__(self, other: GEOSGeometry) -> GEOSGeometry: ...
    def __sub__(self, other: GEOSGeometry) -> GEOSGeometry: ...
    def __xor__(self, other: GEOSGeometry) -> GEOSGeometry: ...
    @property
    def coord_seq(self) -> GEOSCoordSeq | None: ...
    @property
    def geom_type(self) -> str: ...
    @property
    def geom_typeid(self) -> int: ...
    @property
    def num_geom(self) -> int: ...
    @property
    def num_coords(self) -> int: ...
    @property
    def num_points(self) -> int: ...
    @property
    def dims(self) -> int: ...
    def normalize(self) -> None: ...
    @property
    def empty(self) -> bool: ...
    @property
    def hasz(self) -> bool: ...
    @property
    def ring(self) -> bool: ...
    @property
    def simple(self) -> bool: ...
    @property
    def valid(self) -> bool: ...
    @property
    def valid_reason(self) -> str: ...
    def contains(self, other: GEOSGeometry) -> bool: ...
    def covers(self, other: GEOSGeometry) -> bool: ...
    def crosses(self, other: GEOSGeometry) -> bool: ...
    def disjoint(self, other: GEOSGeometry) -> bool: ...
    def equals(self, other: GEOSGeometry) -> bool: ...
    def equals_exact(self, other: GEOSGeometry, tolerance: float = ...) -> bool: ...
    def intersects(self, other: GEOSGeometry) -> bool: ...
    def overlaps(self, other: GEOSGeometry) -> bool: ...
    def relate_pattern(self, other: GEOSGeometry, pattern: str) -> bool: ...
    def touches(self, other: GEOSGeometry) -> bool: ...
    def within(self, other: GEOSGeometry) -> bool: ...
    @property
    def srid(self) -> int | None: ...
    @srid.setter
    def srid(self, srid: int | None) -> None: ...
    @property
    def ewkt(self) -> str: ...
    @property
    def wkt(self) -> str: ...
    @property
    def hex(self) -> bytes: ...
    @property
    def hexewkb(self) -> bytes: ...
    @property
    def json(self) -> str: ...
    geojson: str
    @property
    def wkb(self) -> memoryview: ...
    @property
    def ewkb(self) -> memoryview: ...
    @property
    def kml(self) -> str: ...
    @property
    def prepared(self) -> PreparedGeometry: ...
    @property
    def ogr(self) -> OGRGeometry: ...
    @property
    def srs(self) -> SpatialReference | None: ...
    @property
    def crs(self) -> SpatialReference | None: ...
    ptr: Any
    def transform(self, ct: CoordTransform | SpatialReference | str | int, clone: bool = ...) -> GEOSGeometry: ...
    @property
    def boundary(self) -> GEOSGeometry: ...
    def buffer(self, width: float, quadsegs: int = ...) -> GEOSGeometry: ...
    def buffer_with_style(
        self,
        width: float,
        quadsegs: int = ...,
        end_cap_style: int = ...,
        join_style: int = ...,
        mitre_limit: float = ...,
    ) -> GEOSGeometry: ...
    @property
    def centroid(self) -> Point: ...
    @property
    def convex_hull(self) -> GEOSGeometry: ...
    def difference(self, other: GEOSGeometry) -> GEOSGeometry: ...
    @property
    def envelope(self) -> GEOSGeometry: ...
    def intersection(self, other: GEOSGeometry) -> GEOSGeometry: ...
    @property
    def point_on_surface(self) -> Point: ...
    def relate(self, other: GEOSGeometry) -> str: ...
    def simplify(self, tolerance: float = ..., preserve_topology: bool = ...) -> GEOSGeometry: ...
    def sym_difference(self, other: GEOSGeometry) -> GEOSGeometry: ...
    @property
    def unary_union(self) -> GEOSGeometry: ...
    def union(self, other: GEOSGeometry) -> GEOSGeometry: ...
    @property
    def area(self) -> float: ...
    def distance(self, other: GEOSGeometry) -> float: ...
    @property
    def extent(self) -> tuple[float, float, float, float]: ...
    @property
    def length(self) -> float: ...
    def clone(self: Self) -> Self: ...

class LinearGeometryMixin:
    def interpolate(self, distance: float) -> Point: ...
    def interpolate_normalized(self, distance: float) -> Point: ...
    def project(self, point: Point) -> float: ...
    def project_normalized(self, point: Point) -> float: ...
    @property
    def merged(self) -> GEOSGeometry: ...
    @property
    def closed(self) -> bool: ...

class GEOSGeometry(GEOSGeometryBase, ListMixin):
    def __init__(self, geo_input: Any, srid: int | None = ...) -> None: ...



import sys
from _typeshed import ReadableBuffer, _BufferWithLen
from collections.abc import Callable
from types import ModuleType
from typing import Any, AnyStr, overload
from typing_extensions import TypeAlias

# TODO more precise type for object of hashlib
_Hash: TypeAlias = Any
_DigestMod: TypeAlias = str | Callable[[], _Hash] | ModuleType

trans_5C: bytes
trans_36: bytes

digest_size: None

if sys.version_info >= (3, 8):
    # In reality digestmod has a default value, but the function always throws an error
    # if the argument is not given, so we pretend it is a required argument.
    @overload
    def new(key: bytes | bytearray, msg: ReadableBuffer | None, digestmod: _DigestMod) -> HMAC: ...
    @overload
    def new(key: bytes | bytearray, *, digestmod: _DigestMod) -> HMAC: ...

else:
    def new(key: bytes | bytearray, msg: ReadableBuffer | None = None, digestmod: _DigestMod | None = None) -> HMAC: ...

class HMAC:
    digest_size: int
    block_size: int
    @property
    def name(self) -> str: ...
    if sys.version_info >= (3, 8):
        def __init__(self, key: bytes | bytearray, msg: ReadableBuffer | None = None, digestmod: _DigestMod = "") -> None: ...
    else:
        def __init__(
            self, key: bytes | bytearray, msg: ReadableBuffer | None = None, digestmod: _DigestMod | None = None
        ) -> None: ...

    def update(self, msg: ReadableBuffer) -> None: ...
    def digest(self) -> bytes: ...
    def hexdigest(self) -> str: ...
    def copy(self) -> HMAC: ...

@overload
def compare_digest(__a: ReadableBuffer, __b: ReadableBuffer) -> bool: ...
@overload
def compare_digest(__a: AnyStr, __b: AnyStr) -> bool: ...
def digest(key: _BufferWithLen, msg: ReadableBuffer, digest: _DigestMod) -> bytes: ...



from typing import Any

from django.contrib.gis.db.backends.base.operations import BaseSpatialOperations
from django.contrib.gis.db.backends.utils import SpatialOperator
from django.db.backends.postgresql.operations import DatabaseOperations
from django.db.models import Func
from typing_extensions import Literal

BILATERAL: Literal["bilateral"]

class PostGISOperator(SpatialOperator):
    geography: Any
    raster: bool | Literal["bilateral"]
    def __init__(self, geography: bool = ..., raster: bool | Literal["bilateral"] = ..., **kwargs: Any) -> None: ...
    def check_raster(self, lookup: Any, template_params: Any) -> Any: ...

class ST_Polygon(Func):
    function: str
    def __init__(self, expr: Any) -> None: ...
    @property
    def output_field(self) -> Any: ...

class PostGISOperations(BaseSpatialOperations, DatabaseOperations):
    name: str
    postgis: bool
    geography: bool
    geom_func_prefix: str
    Adapter: Any
    collect: Any
    extent: Any
    extent3d: Any
    length3d: Any
    makeline: Any
    perimeter3d: Any
    unionagg: Any
    gis_operators: Any
    unsupported_functions: Any
    select: str
    select_extent: Any
    @property
    def function_names(self) -> Any: ...
    @property
    def spatial_version(self) -> Any: ...
    def geo_db_type(self, f: Any) -> Any: ...
    def get_distance(self, f: Any, dist_val: Any, lookup_type: Any) -> Any: ...
    def get_geom_placeholder(self, f: Any, value: Any, compiler: Any) -> Any: ...
    def postgis_geos_version(self) -> Any: ...
    def postgis_lib_version(self) -> Any: ...
    def postgis_proj_version(self) -> Any: ...
    def postgis_version(self) -> Any: ...
    def postgis_full_version(self) -> Any: ...
    def postgis_version_tuple(self) -> Any: ...
    def proj_version_tuple(self) -> Any: ...
    def spatial_aggregate_name(self, agg_name: Any) -> Any: ...
    def geometry_columns(self) -> Any: ...
    def spatial_ref_sys(self) -> Any: ...
    def parse_raster(self, value: Any) -> Any: ...
    def distance_expr_for_lookup(self, lhs: Any, rhs: Any, **kwargs: Any) -> Any: ...
    def get_geometry_converter(self, expression: Any) -> Any: ...
    def get_area_att_for_field(self, field: Any) -> Any: ...



from collections.abc import Iterator
from typing import Any

from django.contrib.gis.geos.geometry import GEOSGeometry

class Point(GEOSGeometry):
    has_cs: bool
    def __init__(
        self, x: Any | None = ..., y: Any | None = ..., z: Any | None = ..., srid: Any | None = ...
    ) -> None: ...
    def __iter__(self) -> Iterator[float]: ...
    def __len__(self) -> int: ...
    @property
    def x(self) -> float: ...
    @x.setter
    def x(self, value: float) -> None: ...
    @property
    def y(self) -> float: ...
    @y.setter
    def y(self, value: float) -> None: ...
    @property
    def z(self) -> float | None: ...
    @z.setter
    def z(self, value: float) -> None: ...
    @property
    def coords(self) -> tuple[float, ...]: ...
    @coords.setter
    def coords(self, tup: Any) -> None: ...



from collections import namedtuple
from collections.abc import Iterator, Sequence
from datetime import date as real_date
from typing import Any

from django.template.base import FilterExpression, Parser, Token
from django.template.context import Context
from django.utils.safestring import SafeString

from .base import Node, NodeList
from .library import Library
from .smartif import IfParser, Literal

register: Any

class AutoEscapeControlNode(Node):
    nodelist: NodeList
    setting: bool
    def __init__(self, setting: bool, nodelist: NodeList) -> None: ...

class CommentNode(Node): ...
class CsrfTokenNode(Node): ...

class CycleNode(Node):
    cyclevars: list[FilterExpression]
    variable_name: str | None
    silent: bool
    def __init__(
        self, cyclevars: list[FilterExpression], variable_name: str | None = ..., silent: bool = ...
    ) -> None: ...
    def reset(self, context: Context) -> None: ...

class DebugNode(Node): ...

class FilterNode(Node):
    filter_expr: FilterExpression
    nodelist: NodeList
    def __init__(self, filter_expr: FilterExpression, nodelist: NodeList) -> None: ...

class FirstOfNode(Node):
    vars: list[FilterExpression]
    asvar: str | None
    def __init__(self, variables: list[FilterExpression], asvar: str | None = ...) -> None: ...

class ForNode(Node):
    loopvars: list[str] | str
    sequence: FilterExpression | str
    child_nodelists: Any
    is_reversed: bool
    nodelist_loop: list[str] | NodeList
    nodelist_empty: list[str] | NodeList
    def __init__(
        self,
        loopvars: list[str] | str,
        sequence: FilterExpression | str,
        is_reversed: bool,
        nodelist_loop: list[str] | NodeList,
        nodelist_empty: list[str] | NodeList | None = ...,
    ) -> None: ...

class IfChangedNode(Node):
    nodelist_false: NodeList
    nodelist_true: NodeList
    child_nodelists: Any
    def __init__(self, nodelist_true: NodeList, nodelist_false: NodeList, *varlist: Any) -> None: ...

class IfEqualNode(Node):
    nodelist_false: list[Any] | NodeList
    nodelist_true: list[Any] | NodeList
    var1: FilterExpression | str
    var2: FilterExpression | str
    child_nodelists: Any
    negate: bool
    def __init__(
        self,
        var1: FilterExpression | str,
        var2: FilterExpression | str,
        nodelist_true: list[Any] | NodeList,
        nodelist_false: list[Any] | NodeList,
        negate: bool,
    ) -> None: ...

class IfNode(Node):
    conditions_nodelists: list[tuple[TemplateLiteral | None, NodeList]]
    def __init__(self, conditions_nodelists: list[tuple[TemplateLiteral | None, NodeList]]) -> None: ...
    def __iter__(self) -> Iterator[Node]: ...
    @property
    def nodelist(self) -> NodeList: ...

class LoremNode(Node):
    common: bool
    count: FilterExpression
    method: str
    def __init__(self, count: FilterExpression, method: str, common: bool) -> None: ...

GroupedResult = namedtuple("GroupedResult", ["grouper", "list"])

class RegroupNode(Node):
    expression: FilterExpression
    target: FilterExpression
    var_name: str
    def __init__(self, target: FilterExpression, expression: FilterExpression, var_name: str) -> None: ...
    def resolve_expression(self, obj: dict[str, real_date], context: Context) -> int | str: ...

class LoadNode(Node): ...

class NowNode(Node):
    format_string: str
    asvar: str | None
    def __init__(self, format_string: str, asvar: str | None = ...) -> None: ...

class ResetCycleNode(Node):
    node: CycleNode
    def __init__(self, node: CycleNode) -> None: ...

class SpacelessNode(Node):
    nodelist: NodeList
    def __init__(self, nodelist: NodeList) -> None: ...

class TemplateTagNode(Node):
    mapping: Any
    tagtype: str
    def __init__(self, tagtype: str) -> None: ...

class URLNode(Node):
    view_name: FilterExpression
    args: list[FilterExpression]
    kwargs: dict[str, FilterExpression]
    asvar: str | None
    def __init__(
        self,
        view_name: FilterExpression,
        args: list[FilterExpression],
        kwargs: dict[str, FilterExpression],
        asvar: str | None,
    ) -> None: ...

class VerbatimNode(Node):
    content: SafeString
    def __init__(self, content: SafeString) -> None: ...

class WidthRatioNode(Node):
    val_expr: FilterExpression
    max_expr: FilterExpression
    max_width: FilterExpression
    asvar: str | None
    def __init__(
        self,
        val_expr: FilterExpression,
        max_expr: FilterExpression,
        max_width: FilterExpression,
        asvar: str | None = ...,
    ) -> None: ...

class WithNode(Node):
    nodelist: NodeList
    extra_context: dict[str, Any]
    def __init__(
        self,
        var: str | None,
        name: str | None,
        nodelist: NodeList | Sequence[Node],
        extra_context: dict[str, Any] | None = ...,
    ) -> None: ...

def autoescape(parser: Parser, token: Token) -> AutoEscapeControlNode: ...
def comment(parser: Parser, token: Token) -> CommentNode: ...
def cycle(parser: Parser, token: Token) -> CycleNode: ...
def csrf_token(parser: Parser, token: Token) -> CsrfTokenNode: ...
def debug(parser: Parser, token: Token) -> DebugNode: ...
def do_filter(parser: Parser, token: Token) -> FilterNode: ...
def firstof(parser: Parser, token: Token) -> FirstOfNode: ...
def do_for(parser: Parser, token: Token) -> ForNode: ...
def do_ifequal(parser: Parser, token: Token, negate: bool) -> IfEqualNode: ...
def ifequal(parser: Parser, token: Token) -> IfEqualNode: ...
def ifnotequal(parser: Parser, token: Token) -> IfEqualNode: ...

class TemplateLiteral(Literal):
    text: str
    def __init__(self, value: FilterExpression, text: str) -> None: ...
    def display(self) -> str: ...

class TemplateIfParser(IfParser):
    current_token: TemplateLiteral
    pos: int
    tokens: list[TemplateLiteral]
    error_class: Any
    template_parser: Parser
    def __init__(self, parser: Parser, *args: Any, **kwargs: Any) -> None: ...

def do_if(parser: Parser, token: Token) -> IfNode: ...
def ifchanged(parser: Parser, token: Token) -> IfChangedNode: ...
def find_library(parser: Parser, name: str) -> Library: ...
def load_from_library(library: Library, label: str, names: list[str]) -> Library: ...
def load(parser: Parser, token: Token) -> LoadNode: ...
def lorem(parser: Parser, token: Token) -> LoremNode: ...
def now(parser: Parser, token: Token) -> NowNode: ...
def regroup(parser: Parser, token: Token) -> RegroupNode: ...
def resetcycle(parser: Parser, token: Token) -> ResetCycleNode: ...
def spaceless(parser: Parser, token: Token) -> SpacelessNode: ...
def templatetag(parser: Parser, token: Token) -> TemplateTagNode: ...
def url(parser: Parser, token: Token) -> URLNode: ...
def verbatim(parser: Parser, token: Token) -> VerbatimNode: ...
def widthratio(parser: Parser, token: Token) -> WidthRatioNode: ...
def do_with(parser: Parser, token: Token) -> WithNode: ...



from typing import Optional, Type

from django.db.models.base import Model
from rest_framework.serializers import Serializer

from nwon_django_toolbox.tests.api_client.api_test import ApiTest
from nwon_django_toolbox.tests.helper.dictionary_is_serialized_instance import (
    dictionary_is_serialized_instance,
)
from nwon_django_toolbox.url_helper import detail_url_for_model, list_url_for_model


def check_get_basics(
    model: Model,
    authentication_token: Optional[str] = None,
    return_serializer: Optional[Type[Serializer]] = None,
    needs_authentication: bool = True,
):
    """
    Helper function that checks basic functionality of a get endpoint

    1. Unauthorized on list and detail if needs_authentication == True
    2. Successful request on list and detail with token or without if needs_authentication == True
    (3.) Optional check of responses against serializer
    """

    api_test = ApiTest()

    if needs_authentication:
        list_url = list_url_for_model(model)
        non_existent_detail_url = detail_url_for_model(model, "some-non-existing-id")

        api_test.get_unauthorized(list_url)
        api_test.get_unauthorized(non_existent_detail_url)

        if authentication_token:
            api_test.set_bearer_token(authentication_token)
            __basic_get_test(api_test, model, return_serializer)
    else:
        __basic_get_test(api_test, model, return_serializer)


def __basic_get_test(
    api_test: ApiTest,
    model: Model,
    return_serializer: Optional[Type[Serializer]] = None,
):
    list_url = list_url_for_model(model)
    non_existent_detail_url = detail_url_for_model(model, "some-non-existing-id")
    detail_url = detail_url_for_model(model)

    api_test.get_not_found(non_existent_detail_url)

    response_list = api_test.get_successful(list_url)

    response_detail = api_test.get_successful(detail_url)

    if return_serializer:
        assert dictionary_is_serialized_instance(
            instance=model,
            serializer=return_serializer,
            response=response_list["results"][0],
        )

        assert dictionary_is_serialized_instance(
            instance=model,
            serializer=return_serializer,
            response=response_detail,
        )


__all__ = [
    "check_get_basics",
]
